from org.transcrypt.stubs.browser import __pragma__
__pragma__('alias', "jQuery", "$")
__pragma__('skip')

# it is ignored on transcrypt
window = jQuery = console = document = localStorage = URL =\
    sessionStorage = this = FileReader = JSON = WebSocket = js_undefined = navigator = setTimeout = __new__ = Date = 0

__pragma__('noskip')


__pragma__('kwargs')


class Ajax():
    def __init__(self, http_address):
        self.remote_address = http_address
        self._headers = {
            'Cache-Control': 'no-store, must-revalidate, no-cache, max-age=0',
            'phanterpwa-application': window.PhanterPWA.CONFIG['PROJECT']['name'],
            'phanterpwa-application-version': window.PhanterPWA.CONFIG['PROJECT']['version']
        }

    def _process_args(self, args):
        s_args = ""
        if isinstance(args, list):
            s_args = "/".join(args)
        if s_args != "":
            s_args = "{0}/".format(s_args)
        return s_args

    def _serialize_vars(self, _vars):
        ISTYPEOF = __pragma__("js", "{}", "_vars instanceof FormData")
        if _vars is None or _vars is js_undefined:
            return ""
        elif ISTYPEOF:
            return ""
        elif isinstance(_vars, dict):
            __pragma__('jsiter')
            jsdict = {}
            __pragma__('nojsiter')
            for x in _vars.keys():
                if _vars[x] is not None and _vars[x] is not js_undefined:
                    jsdict[x] = _vars[x]
            return "?{0}".format(jQuery.param(jsdict))
        else:
            t = jQuery.param(_vars)
            if t is not None or t is not js_undefined:
                return "?{0}".format(t)
            else:
                return ""

    def GET(self, *url_args, **parameters):
        _headers = dict(**self._headers)
        _datatype = parameters.get("datatype", "json")
        _url_vars = parameters.get("url_vars", None)
        _complete = parameters.get("complete", None)
        _crossDomain = parameters.get("crossDomain", True)
        _error = parameters.get("error", None)
        _success = parameters.get("success", None)
        _user_headers = parameters.get("headers", {})
        for x in _user_headers.keys():
            _headers[x] = _user_headers[x]
        _all_args = self._process_args(url_args)
        _all_vars = self._serialize_vars(_url_vars)
        _url = "{0}/{1}{2}".format(self.remote_address, _all_args, _all_vars)

        __pragma__('jsiter')
        ajax_param = {
            'url': _url,
            'type': "GET",
            'complete': lambda a, b, c: _complete(a, b, c),
            'success': _success,
            'error': _error,
            'datatype': _datatype,
            'crossDomain': _crossDomain,
            'headers': _headers
        }
        client_token = window.PhanterPWA.get_client_token()
        authorization = window.PhanterPWA.get_authorization()
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None:
            ajax_param['headers']['phanterpwa-authorization'] = authorization

        __pragma__('nojsiter')
        jQuery.ajax(ajax_param)

    def DELETE(self, *url_args, **parameters):
        _headers = dict(**self._headers)
        _datatype = parameters.get("datatype", "json")
        _url_vars = parameters.get("url_vars", None)
        _complete = parameters.get("complete", None)
        _crossDomain = parameters.get("crossDomain", True)
        _error = parameters.get("error", None)
        _success = parameters.get("success", None)
        _user_headers = parameters.get("headers", {})
        for x in _user_headers.keys():
            _headers[x] = _user_headers[x]
        _all_args = self._process_args(url_args)
        _all_vars = self._serialize_vars(_url_vars)
        _current_uri = "/{0}{1}".format()
        _url = "{0}/{1}{2}".format(self.remote_address, _all_args, _all_vars)

        __pragma__('jsiter')
        ajax_param = {
            'url': _url,
            'type': "DELETE",
            'complete': lambda a, b, c: _complete(a, b, c),
            'success': _success,
            'error': _error,
            'datatype': _datatype,
            'crossDomain': _crossDomain,
            'headers': _headers
        }
        client_token = window.PhanterPWA.get_client_token()
        authorization = window.PhanterPWA.get_authorization()
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None:
            ajax_param['headers']['phanterpwa-authorization'] = authorization

        __pragma__('nojsiter')
        jQuery.ajax(ajax_param)

    def POST(self, *url_args, **parameters):
        _headers = dict(**self._headers)
        _datatype = parameters.get("datatype", "json")
        _url_vars = parameters.get("url_vars", None)
        _complete = parameters.get("complete", None)
        _form_data = parameters.get("form_data", None)
        _crossDomain = parameters.get("crossDomain", True)
        _error = parameters.get("error", None)
        _success = parameters.get("success", None)
        _user_headers = parameters.get("headers", {})
        for x in _user_headers.keys():
            _headers[x] = _user_headers[x]
        _all_args = self._process_args(url_args)
        _all_vars = self._serialize_vars(_url_vars)
        _url = "{0}/{1}{2}".format(self.remote_address, _all_args, _all_vars)

        __pragma__('jsiter')
        ajax_param = {
            'url': _url,
            'type': "POST",
            'data': _form_data,
            'complete': lambda a, b, c: _complete(a, b, c),
            'success': _success,
            'error': _error,
            'datatype': _datatype,
            'crossDomain': _crossDomain,
            'headers': _headers
        }
        client_token = window.PhanterPWA.get_client_token()
        authorization = window.PhanterPWA.get_authorization()
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None:
            ajax_param['headers']['phanterpwa-authorization'] = authorization
        ISTYPEOF = __pragma__("js", "{}", "form_data instanceof FormData")
        if ISTYPEOF:
            ajax_param['processData'] = False
            ajax_param['contentType'] = False
        __pragma__('nojsiter')
        jQuery.ajax(ajax_param)

    def PUT(self, *url_args, **parameters):
        _headers = dict(**self._headers)
        _datatype = parameters.get("datatype", "json")
        _url_vars = parameters.get("url_vars", None)
        _complete = parameters.get("complete", None)
        _form_data = parameters.get("form_data", None)
        _crossDomain = parameters.get("crossDomain", True)
        _error = parameters.get("error", None)
        _success = parameters.get("success", None)
        _user_headers = parameters.get("headers", {})
        for x in _user_headers.keys():
            _headers[x] = _user_headers[x]
        _all_args = self._process_args(url_args)
        _all_vars = self._serialize_vars(_url_vars)
        _url = "{0}/{1}{2}".format(self.remote_address, _all_args, _all_vars)

        __pragma__('jsiter')
        ajax_param = {
            'url': _url,
            'type': "PUT",
            'data': _form_data,
            'complete': lambda a, b, c: _complete(a, b, c),
            'success': _success,
            'error': _error,
            'datatype': _datatype,
            'crossDomain': _crossDomain,
            'headers': _headers
        }
        client_token = window.PhanterPWA.get_client_token()
        authorization = window.PhanterPWA.get_authorization()
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None:
            ajax_param['headers']['phanterpwa-authorization'] = authorization
        ISTYPEOF = __pragma__("js", "{}", "form_data instanceof FormData")
        if ISTYPEOF:
            ajax_param['processData'] = False
            ajax_param['contentType'] = False
        __pragma__('nojsiter')
        jQuery.ajax(ajax_param)

    def getClientToken(self, callback=None):
        def onComplete(data, ajax_status):
            if ajax_status == "success":
                auth_user = data.responseJSON.auth_user
                if (auth_user == "anonymous") or (auth_user == "logout"):
                    sessionStorage.removeItem("phanterpwa-authorization")
                    sessionStorage.removeItem("auth_user")
                    localStorage.removeItem("phanterpwa-authorization")
                    localStorage.removeItem("auth_user")
                client_token = data.responseJSON.client_token
                url_token = data.responseJSON.url_token
                if (url_token is not js_undefined):
                    localStorage.setItem('phanterpwa-url-token', url_token)

                if (client_token is not js_undefined):
                    localStorage.setItem("phanterpwa-client-token", client_token)
                    if callback is not None and callback is not js_undefined:
                        callback(data, ajax_status)
            else:
                if data.status == 0:
                    console.info("Server Problem!")
                elif data.status == 400:
                    sessionStorage.js_clear()
                    localStorage.js_clear()

        client_token = localStorage.getItem("phanterpwa-client-token")
        session_authorization = sessionStorage.getItem("phanterpwa-authorization")

        if client_token is None or client_token is js_undefined:
            sessionStorage.removeItem("phanterpwa-authorization")
            sessionStorage.removeItem("auth_user")
        if session_authorization is None or session_authorization is js_undefined:
            local_authorization = localStorage.getItem("phanterpwa-authorization")
            if local_authorization is None or local_authorization is js_undefined:
                sessionStorage.removeItem("phanterpwa-authorization")
                sessionStorage.removeItem("auth_user")
                localStorage.removeItem("phanterpwa-authorization")
                localStorage.removeItem("auth_user")

        return self.GET(**{'url_args': ['api', 'client'], 'onComplete': onComplete})

    def reSignCredentials(self):
        date_stamp = __new__(Date().getTime())
        last_resing = sessionStorage.getItem("last_resing")
        authorization = sessionStorage.getItem("phanterpwa-authorization")
        auth_user = sessionStorage.getItem("auth_user")
        if auth_user is None or auth_user is js_undefined:
            auth_user = localStorage.getItem("auth_user")
        lets_go_resign = False
        if last_resing is not None and last_resing is not js_undefined:
            timeout_last_resign = window.PhanterPWA.CONFIG['CONFIGJS']['timeout_to_resign'] * 1000
            if (int(last_resing) + int(timeout_last_resign)) < int(date_stamp):
                lets_go_resign = True
                sessionStorage.setItem("last_resing", date_stamp)
        else:
            sessionStorage.setItem("last_resing", date_stamp)
            lets_go_resign = True

        def onComplete(data, ajax_status):
            json = data.responseJSON
            if ajax_status == "success":
                if json.url_token is not None and json.url_token is not js_undefined:
                    localStorage.setItem('phanterpwa-url-token', json.url_token)
                if json.client_token is not None and json.client_token is not js_undefined:
                    localStorage.setItem('phanterpwa-client-token', json.client_token)
                if json.authorization is not None and json.authorization is not js_undefined:
                    if auth_user["remember_me"] is True:
                        localStorage.setItem('phanterpwa-authorization', json.authorization)
                    else:
                        localStorage.setItem('phanterpwa-authorization', json.authorization)
        if lets_go_resign and authorization is not None and authorization is not js_undefined:
            if auth_user is not None and auth_user is not js_undefined:
                self.GET(**{
                    "url_args": ["api", "resigncredentials"],
                    "url_vars": {"_": date_stamp},
                    "onComplete": onComplete
                })


class Websocket():
    def __init__(self, websocket_address):
        self.remote_address = websocket_address
        self._headers = {
            'Cache-Control': 'no-store, must-revalidate, no-cache, max-age=0',
            'phanterpwa-application': window.PhanterPWA.CONFIG['PROJECT']['name'],
            'phanterpwa-application-version': window.PhanterPWA.CONFIG['PROJECT']['version']
        }
        self._opened = False
        self.websocket_address = websocket_address
        self.comulative_time = 300
        self.last_ws = None
        self.manual_connection = False
        self.manual_close = False
        self.onClose = None
        self.onError = None
        self.onMessage = None
        self.onOpen = None
        self.show_alert_reconnect = False
        setTimeout(lambda: self.connect(), self.comulative_time)

    def send(self, message):
        if self._opened:
            self._ws.send(message)

    def close(self):
        self.manual_close = True
        self._ws.close()

    def _onClose(self, evt):
        self._opened = False
        if window.PhanterPWA.DEBUG:
            console.info("Closing websocket")
        if not self.manual_close:
            setTimeout(lambda: self.start(), self.comulative_time)
        else:
            self.manual_close = False
        if callable(self.onClose):
            self.onClose(evt)

    def _onError(self, evt):
        if window.PhanterPWA.DEBUG:
            console.info("Error on websocket", evt)
        if self.comulative_time > 9000 and not self.manual_connection:
                window.PhanterPWA.flash("Lost server connection!")
                self.manual_connection = True
        else:
            self.comulative_time = self.comulative_time + 1000
        if callable(self.onError):
            self.onError(evt)

    def _onMessage(self, evt):
        if window.PhanterPWA.DEBUG:
            console.info(evt.data)
        if callable(self.onMessage):
            self.onMessage(evt)

    def _onOpen(self, evt):
        self.comulative_time = 0
        self._opened = True
        if window.PhanterPWA.DEBUG:
            console.info("Opening websocket")
        self.send("command_online")
        if callable(self.onOpen):
            self.onOpen(evt)

    def _binds(self):
        self._ws.onopen = lambda evt: self._onOpen(evt)
        self._ws.onmessage = lambda evt: self._onMessage(evt)
        self._ws.onerror = lambda evt: self._onError(evt)
        self._ws.onclose = lambda evt: self._onClose(evt)

    def connect(self):
        if not self.manual_connection:
            self._ws = __new__(WebSocket(self.websocket_address))
            self._authorization = window.PhanterPWA.get_authorization()
            self._config = window.PhanterPWA.CONFIG
            self._binds()
        else:
            if callable(self.onConectionProblem):
                self.onConectionProblem(self)
            else:
                console.alert("Connection problem! You need to reconnect!")

    def reconnect(self):
        self.manual_connection = False
        self.comulative_time = 0
        self._ws = __new__(WebSocket(self.websocket_address))
        self._authorization = window.PhanterPWA.get_authorization()
        self._config = window.PhanterPWA.CONFIG
        self._binds()

    def _process_args(self, args):
        s_args = ""
        if isinstance(args, list):
            return args
        else:
            return None

    def _serialize_vars(self, _vars):
        ISTYPEOF = __pragma__("js", "{}", "_vars instanceof FormData")
        if _vars is None or _vars is js_undefined:
            return ""
        elif ISTYPEOF:
            return ""
        elif isinstance(_vars, dict):
            __pragma__('jsiter')
            jsdict = {}
            __pragma__('nojsiter')
            for x in _vars.keys():
                if _vars[x] is not None and _vars[x] is not js_undefined:
                    jsdict[x] = _vars[x]
            return jsdict
        else:
            t = jQuery.param(_vars)
            if t is not None or t is not js_undefined:
                return t
            else:
                return ""

    def GET(self, *url_args, **parameters):
        _headers = dict(**self._headers)
        _datatype = parameters.get("datatype", "json")
        _url_vars = parameters.get("url_vars", None)
        _complete = parameters.get("complete", None)
        _crossDomain = parameters.get("crossDomain", True)
        _error = parameters.get("error", None)
        _success = parameters.get("success", None)
        _user_headers = parameters.get("headers", {})
        for x in _user_headers.keys():
            _headers[x] = _user_headers[x]
        _all_args = self._process_args(url_args)
        _all_vars = self._serialize_vars(_url_vars)
        _url = "{0}/{1}{2}".format(self.remote_address, _all_args, _all_vars)

        __pragma__('jsiter')
        ajax_param = {
            'url': _url,
            'type': "GET",
            'complete': lambda a, b, c: _complete(a, b, c),
            'success': _success,
            'error': _error,
            'datatype': _datatype,
            'crossDomain': _crossDomain,
            'headers': _headers
        }
        client_token = window.PhanterPWA.get_client_token()
        authorization = window.PhanterPWA.get_authorization()
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None:
            ajax_param['headers']['phanterpwa-authorization'] = authorization

        __pragma__('nojsiter')
        self.send(ajax_param)

    def DELETE(self, *url_args, **parameters):
        _headers = dict(**self._headers)
        _datatype = parameters.get("datatype", "json")
        _url_vars = parameters.get("url_vars", None)
        _complete = parameters.get("complete", None)
        _crossDomain = parameters.get("crossDomain", True)
        _error = parameters.get("error", None)
        _success = parameters.get("success", None)
        _user_headers = parameters.get("headers", {})
        for x in _user_headers.keys():
            _headers[x] = _user_headers[x]
        _all_args = self._process_args(url_args)
        _all_vars = self._serialize_vars(_url_vars)
        _current_uri = "/{0}{1}".format()
        _url = "{0}/{1}{2}".format(self.remote_address, _all_args, _all_vars)

        __pragma__('jsiter')
        ajax_param = {
            'url': _url,
            'type': "DELETE",
            'complete': lambda a, b, c: _complete(a, b, c),
            'success': _success,
            'error': _error,
            'datatype': _datatype,
            'crossDomain': _crossDomain,
            'headers': _headers
        }
        client_token = window.PhanterPWA.get_client_token()
        authorization = window.PhanterPWA.get_authorization()
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None:
            ajax_param['headers']['phanterpwa-authorization'] = authorization

        __pragma__('nojsiter')
        self.send(ajax_param)

    def POST(self, *url_args, **parameters):
        _headers = dict(**self._headers)
        _datatype = parameters.get("datatype", "json")
        _url_vars = parameters.get("url_vars", None)
        _complete = parameters.get("complete", None)
        _form_data = parameters.get("form_data", None)
        _crossDomain = parameters.get("crossDomain", True)
        _error = parameters.get("error", None)
        _success = parameters.get("success", None)
        _user_headers = parameters.get("headers", {})
        for x in _user_headers.keys():
            _headers[x] = _user_headers[x]
        _all_args = self._process_args(url_args)
        _all_vars = self._serialize_vars(_url_vars)
        _url = "{0}/{1}{2}".format(self.remote_address, _all_args, _all_vars)

        __pragma__('jsiter')
        ajax_param = {
            'url': _url,
            'type': "POST",
            'data': _form_data,
            'complete': lambda a, b, c: _complete(a, b, c),
            'success': _success,
            'error': _error,
            'datatype': _datatype,
            'crossDomain': _crossDomain,
            'headers': _headers
        }
        client_token = window.PhanterPWA.get_client_token()
        authorization = window.PhanterPWA.get_authorization()
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None:
            ajax_param['headers']['phanterpwa-authorization'] = authorization
        ISTYPEOF = __pragma__("js", "{}", "form_data instanceof FormData")
        if ISTYPEOF:
            ajax_param['processData'] = False
            ajax_param['contentType'] = False
        __pragma__('nojsiter')
        self.send(ajax_param)

    def PUT(self, *url_args, **parameters):
        _headers = dict(**self._headers)
        _datatype = parameters.get("datatype", "json")
        _url_vars = parameters.get("url_vars", None)
        _complete = parameters.get("complete", None)
        _form_data = parameters.get("form_data", None)
        _crossDomain = parameters.get("crossDomain", True)
        _error = parameters.get("error", None)
        _success = parameters.get("success", None)
        _user_headers = parameters.get("headers", {})
        for x in _user_headers.keys():
            _headers[x] = _user_headers[x]
        _all_args = self._process_args(url_args)
        _all_vars = self._serialize_vars(_url_vars)
        _url = "{0}/{1}{2}".format(self.remote_address, _all_args, _all_vars)

        __pragma__('jsiter')
        ajax_param = {
            'url': _url,
            'type': "PUT",
            'data': _form_data,
            'complete': lambda a, b, c: _complete(a, b, c),
            'success': _success,
            'error': _error,
            'datatype': _datatype,
            'crossDomain': _crossDomain,
            'headers': _headers
        }
        client_token = window.PhanterPWA.get_client_token()
        authorization = window.PhanterPWA.get_authorization()
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None:
            ajax_param['headers']['phanterpwa-authorization'] = authorization
        ISTYPEOF = __pragma__("js", "{}", "form_data instanceof FormData")
        if ISTYPEOF:
            ajax_param['processData'] = False
            ajax_param['contentType'] = False
        __pragma__('nojsiter')
        self.send(ajax_param)

    def getClientToken(self, callback=None):
        def onComplete(data, ajax_status):
            if ajax_status == "success":
                auth_user = data.responseJSON.auth_user
                if (auth_user == "anonymous") or (auth_user == "logout"):
                    sessionStorage.removeItem("phanterpwa-authorization")
                    sessionStorage.removeItem("auth_user")
                    localStorage.removeItem("phanterpwa-authorization")
                    localStorage.removeItem("auth_user")
                client_token = data.responseJSON.client_token
                url_token = data.responseJSON.url_token
                if (url_token is not js_undefined):
                    localStorage.setItem('phanterpwa-url-token', url_token)

                if (client_token is not js_undefined):
                    localStorage.setItem("phanterpwa-client-token", client_token)
                    if callback is not None and callback is not js_undefined:
                        callback(data, ajax_status)
            else:
                if data.status == 0:
                    console.info("Server Problem!")
                elif data.status == 400:
                    sessionStorage.js_clear()
                    localStorage.js_clear()

        client_token = localStorage.getItem("phanterpwa-client-token")
        session_authorization = sessionStorage.getItem("phanterpwa-authorization")

        if client_token is None or client_token is js_undefined:
            sessionStorage.removeItem("phanterpwa-authorization")
            sessionStorage.removeItem("auth_user")
        if session_authorization is None or session_authorization is js_undefined:
            local_authorization = localStorage.getItem("phanterpwa-authorization")
            if local_authorization is None or local_authorization is js_undefined:
                sessionStorage.removeItem("phanterpwa-authorization")
                sessionStorage.removeItem("auth_user")
                localStorage.removeItem("phanterpwa-authorization")
                localStorage.removeItem("auth_user")

        return self.GET(**{'url_args': ['api', 'client'], 'onComplete': onComplete})

    def reSignCredentials(self):
        date_stamp = __new__(Date().getTime())
        last_resing = sessionStorage.getItem("last_resing")
        authorization = sessionStorage.getItem("phanterpwa-authorization")
        auth_user = sessionStorage.getItem("auth_user")
        if auth_user is None or auth_user is js_undefined:
            auth_user = localStorage.getItem("auth_user")
        lets_go_resign = False
        if last_resing is not None and last_resing is not js_undefined:
            timeout_last_resign = window.PhanterPWA.CONFIG['CONFIGJS']['timeout_to_resign'] * 1000
            if (int(last_resing) + int(timeout_last_resign)) < int(date_stamp):
                lets_go_resign = True
                sessionStorage.setItem("last_resing", date_stamp)
        else:
            sessionStorage.setItem("last_resing", date_stamp)
            lets_go_resign = True

        def onComplete(data, ajax_status):
            json = data.responseJSON
            if ajax_status == "success":
                if json.url_token is not None and json.url_token is not js_undefined:
                    localStorage.setItem('phanterpwa-url-token', json.url_token)
                if json.client_token is not None and json.client_token is not js_undefined:
                    localStorage.setItem('phanterpwa-client-token', json.client_token)
                if json.authorization is not None and json.authorization is not js_undefined:
                    if auth_user["remember_me"] is True:
                        localStorage.setItem('phanterpwa-authorization', json.authorization)
                    else:
                        localStorage.setItem('phanterpwa-authorization', json.authorization)
        if lets_go_resign and authorization is not None and authorization is not js_undefined:
            if auth_user is not None and auth_user is not js_undefined:
                self.GET(**{
                    "url_args": ["api", "resigncredentials"],
                    "url_vars": {"_": date_stamp},
                    "onComplete": onComplete
                })

__pragma__('nokwargs')



import phanterpwa.frontend.gatehandler as gatehandler
import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.components.left_bar as left_bar
from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = M = js_undefined = window = setTimeout=\
    __new__ = FormData = console = localStorage = document = this = 0
__pragma__('noskip')

DIV = helpers.XmlConstructor.tagger("div")
LABEL = helpers.XmlConstructor.tagger("label")
I = helpers.XmlConstructor.tagger("i")
H2 = helpers.XmlConstructor.tagger("h2")
FORM = helpers.XmlConstructor.tagger("form")
SPAN = helpers.XmlConstructor.tagger("span")
IMG = helpers.XmlConstructor.tagger("img", True)
HR = helpers.XmlConstructor.tagger("hr", True)
UL = helpers.XmlConstructor.tagger("ul")
LI = helpers.XmlConstructor.tagger("li")
INPUT = helpers.XmlConstructor.tagger("input", True)
P = helpers.XmlConstructor.tagger("p")
A = helpers.XmlConstructor.tagger("a")
TABLE = helpers.XmlConstructor.tagger("table")
TD = helpers.XmlConstructor.tagger("td")
TR = helpers.XmlConstructor.tagger("tr")
I18N = helpers.I18N
CONCATENATE = helpers.CONCATENATE
XML = helpers.XML
XSECTION = helpers.XSECTION
list_icons = [
    "fab fa-500px",
    "fab fa-accessible-icon",
    "fab fa-accusoft",
    "fab fa-acquisitions-incorporated",
    "fas fa-ad",
    "fas fa-address-book",
    "far fa-address-book",
    "fas fa-address-card",
    "far fa-address-card",
    "fas fa-adjust",
    "fab fa-adn",
    "fab fa-adobe",
    "fab fa-adversal",
    "fab fa-affiliatetheme",
    "fas fa-air-freshener",
    "fab fa-airbnb",
    "fab fa-algolia",
    "fas fa-align-center",
    "fas fa-align-justify",
    "fas fa-align-left",
    "fas fa-align-right",
    "fab fa-alipay",
    "fas fa-allergies",
    "fab fa-amazon",
    "fab fa-amazon-pay",
    "fas fa-ambulance",
    "fas fa-american-sign-language-interpreting",
    "fab fa-amilia",
    "fas fa-anchor",
    "fab fa-android",
    "fab fa-angellist",
    "fas fa-angle-double-down",
    "fas fa-angle-double-left",
    "fas fa-angle-double-right",
    "fas fa-angle-double-up",
    "fas fa-angle-down",
    "fas fa-angle-left",
    "fas fa-angle-right",
    "fas fa-angle-up",
    "fas fa-angry",
    "far fa-angry",
    "fab fa-angrycreative",
    "fab fa-angular",
    "fas fa-ankh",
    "fab fa-app-store",
    "fab fa-app-store-ios",
    "fab fa-apper",
    "fab fa-apple",
    "fas fa-apple-alt",
    "fab fa-apple-pay",
    "fas fa-archive",
    "fas fa-archway",
    "fas fa-arrow-alt-circle-down",
    "far fa-arrow-alt-circle-down",
    "fas fa-arrow-alt-circle-left",
    "far fa-arrow-alt-circle-left",
    "fas fa-arrow-alt-circle-right",
    "far fa-arrow-alt-circle-right",
    "fas fa-arrow-alt-circle-up",
    "far fa-arrow-alt-circle-up",
    "fas fa-arrow-circle-down",
    "fas fa-arrow-circle-left",
    "fas fa-arrow-circle-right",
    "fas fa-arrow-circle-up",
    "fas fa-arrow-down",
    "fas fa-arrow-left",
    "fas fa-arrow-right",
    "fas fa-arrow-up",
    "fas fa-arrows-alt",
    "fas fa-arrows-alt-h",
    "fas fa-arrows-alt-v",
    "fab fa-artstation",
    "fas fa-assistive-listening-systems",
    "fas fa-asterisk",
    "fab fa-asymmetrik",
    "fas fa-at",
    "fas fa-atlas",
    "fab fa-atlassian",
    "fas fa-atom",
    "fab fa-audible",
    "fas fa-audio-description",
    "fab fa-autoprefixer",
    "fab fa-avianex",
    "fab fa-aviato",
    "fas fa-award",
    "fab fa-aws",
    "fas fa-baby",
    "fas fa-baby-carriage",
    "fas fa-backspace",
    "fas fa-backward",
    "fas fa-bacon",
    "fas fa-balance-scale",
    "fas fa-balance-scale-left",
    "fas fa-balance-scale-right",
    "fas fa-ban",
    "fas fa-band-aid",
    "fab fa-bandcamp",
    "fas fa-barcode",
    "fas fa-bars",
    "fas fa-baseball-ball",
    "fas fa-basketball-ball",
    "fas fa-bath",
    "fas fa-battery-empty",
    "fas fa-battery-full",
    "fas fa-battery-half",
    "fas fa-battery-quarter",
    "fas fa-battery-three-quarters",
    "fab fa-battle-net",
    "fas fa-bed",
    "fas fa-beer",
    "fab fa-behance",
    "fab fa-behance-square",
    "fas fa-bell",
    "far fa-bell",
    "fas fa-bell-slash",
    "far fa-bell-slash",
    "fas fa-bezier-curve",
    "fas fa-bible",
    "fas fa-bicycle",
    "fas fa-biking",
    "fab fa-bimobject",
    "fas fa-binoculars",
    "fas fa-biohazard",
    "fas fa-birthday-cake",
    "fab fa-bitbucket",
    "fab fa-bitcoin",
    "fab fa-bity",
    "fab fa-black-tie",
    "fab fa-blackberry",
    "fas fa-blender",
    "fas fa-blender-phone",
    "fas fa-blind",
    "fas fa-blog",
    "fab fa-blogger",
    "fab fa-blogger-b",
    "fab fa-bluetooth",
    "fab fa-bluetooth-b",
    "fas fa-bold",
    "fas fa-bolt",
    "fas fa-bomb",
    "fas fa-bone",
    "fas fa-bong",
    "fas fa-book",
    "fas fa-book-dead",
    "fas fa-book-medical",
    "fas fa-book-open",
    "fas fa-book-reader",
    "fas fa-bookmark",
    "far fa-bookmark",
    "fab fa-bootstrap",
    "fas fa-border-all",
    "fas fa-border-none",
    "fas fa-border-style",
    "fas fa-bowling-ball",
    "fas fa-box",
    "fas fa-box-open",
    "fas fa-boxes",
    "fas fa-braille",
    "fas fa-brain",
    "fas fa-bread-slice",
    "fas fa-briefcase",
    "fas fa-briefcase-medical",
    "fas fa-broadcast-tower",
    "fas fa-broom",
    "fas fa-brush",
    "fab fa-btc",
    "fab fa-buffer",
    "fas fa-bug",
    "fas fa-building",
    "far fa-building",
    "fas fa-bullhorn",
    "fas fa-bullseye",
    "fas fa-burn",
    "fab fa-buromobelexperte",
    "fas fa-bus",
    "fas fa-bus-alt",
    "fas fa-business-time",
    "fab fa-buy-n-large",
    "fab fa-buysellads",
    "fas fa-calculator",
    "fas fa-calendar",
    "far fa-calendar",
    "fas fa-calendar-alt",
    "far fa-calendar-alt",
    "fas fa-calendar-check",
    "far fa-calendar-check",
    "fas fa-calendar-day",
    "fas fa-calendar-minus",
    "far fa-calendar-minus",
    "fas fa-calendar-plus",
    "far fa-calendar-plus",
    "fas fa-calendar-times",
    "far fa-calendar-times",
    "fas fa-calendar-week",
    "fas fa-camera",
    "fas fa-camera-retro",
    "fas fa-campground",
    "fab fa-canadian-maple-leaf",
    "fas fa-candy-cane",
    "fas fa-cannabis",
    "fas fa-capsules",
    "fas fa-car",
    "fas fa-car-alt",
    "fas fa-car-battery",
    "fas fa-car-crash",
    "fas fa-car-side",
    "fas fa-caret-down",
    "fas fa-caret-left",
    "fas fa-caret-right",
    "fas fa-caret-square-down",
    "far fa-caret-square-down",
    "fas fa-caret-square-left",
    "far fa-caret-square-left",
    "fas fa-caret-square-right",
    "far fa-caret-square-right",
    "fas fa-caret-square-up",
    "far fa-caret-square-up",
    "fas fa-caret-up",
    "fas fa-carrot",
    "fas fa-cart-arrow-down",
    "fas fa-cart-plus",
    "fas fa-cash-register",
    "fas fa-cat",
    "fab fa-cc-amazon-pay",
    "fab fa-cc-amex",
    "fab fa-cc-apple-pay",
    "fab fa-cc-diners-club",
    "fab fa-cc-discover",
    "fab fa-cc-jcb",
    "fab fa-cc-mastercard",
    "fab fa-cc-paypal",
    "fab fa-cc-stripe",
    "fab fa-cc-visa",
    "fab fa-centercode",
    "fab fa-centos",
    "fas fa-certificate",
    "fas fa-chair",
    "fas fa-chalkboard",
    "fas fa-chalkboard-teacher",
    "fas fa-charging-station",
    "fas fa-chart-area",
    "fas fa-chart-bar",
    "far fa-chart-bar",
    "fas fa-chart-line",
    "fas fa-chart-pie",
    "fas fa-check",
    "fas fa-check-circle",
    "far fa-check-circle",
    "fas fa-check-double",
    "fas fa-check-square",
    "far fa-check-square",
    "fas fa-cheese",
    "fas fa-chess",
    "fas fa-chess-bishop",
    "fas fa-chess-board",
    "fas fa-chess-king",
    "fas fa-chess-knight",
    "fas fa-chess-pawn",
    "fas fa-chess-queen",
    "fas fa-chess-rook",
    "fas fa-chevron-circle-down",
    "fas fa-chevron-circle-left",
    "fas fa-chevron-circle-right",
    "fas fa-chevron-circle-up",
    "fas fa-chevron-down",
    "fas fa-chevron-left",
    "fas fa-chevron-right",
    "fas fa-chevron-up",
    "fas fa-child",
    "fab fa-chrome",
    "fab fa-chromecast",
    "fas fa-church",
    "fas fa-circle",
    "far fa-circle",
    "fas fa-circle-notch",
    "fas fa-city",
    "fas fa-clinic-medical",
    "fas fa-clipboard",
    "far fa-clipboard",
    "fas fa-clipboard-check",
    "fas fa-clipboard-list",
    "fas fa-clock",
    "far fa-clock",
    "fas fa-clone",
    "far fa-clone",
    "fas fa-closed-captioning",
    "far fa-closed-captioning",
    "fas fa-cloud",
    "fas fa-cloud-download-alt",
    "fas fa-cloud-meatball",
    "fas fa-cloud-moon",
    "fas fa-cloud-moon-rain",
    "fas fa-cloud-rain",
    "fas fa-cloud-showers-heavy",
    "fas fa-cloud-sun",
    "fas fa-cloud-sun-rain",
    "fas fa-cloud-upload-alt",
    "fab fa-cloudscale",
    "fab fa-cloudsmith",
    "fab fa-cloudversify",
    "fas fa-cocktail",
    "fas fa-code",
    "fas fa-code-branch",
    "fab fa-codepen",
    "fab fa-codiepie",
    "fas fa-coffee",
    "fas fa-cog",
    "fas fa-cogs",
    "fas fa-coins",
    "fas fa-columns",
    "fas fa-comment",
    "far fa-comment",
    "fas fa-comment-alt",
    "far fa-comment-alt",
    "fas fa-comment-dollar",
    "fas fa-comment-dots",
    "far fa-comment-dots",
    "fas fa-comment-medical",
    "fas fa-comment-slash",
    "fas fa-comments",
    "far fa-comments",
    "fas fa-comments-dollar",
    "fas fa-compact-disc",
    "fas fa-compass",
    "far fa-compass",
    "fas fa-compress",
    "fas fa-compress-arrows-alt",
    "fas fa-concierge-bell",
    "fab fa-confluence",
    "fab fa-connectdevelop",
    "fab fa-contao",
    "fas fa-cookie",
    "fas fa-cookie-bite",
    "fas fa-copy",
    "far fa-copy",
    "fas fa-copyright",
    "far fa-copyright",
    "fab fa-cotton-bureau",
    "fas fa-couch",
    "fab fa-cpanel",
    "fab fa-creative-commons",
    "fab fa-creative-commons-by",
    "fab fa-creative-commons-nc",
    "fab fa-creative-commons-nc-eu",
    "fab fa-creative-commons-nc-jp",
    "fab fa-creative-commons-nd",
    "fab fa-creative-commons-pd",
    "fab fa-creative-commons-pd-alt",
    "fab fa-creative-commons-remix",
    "fab fa-creative-commons-sa",
    "fab fa-creative-commons-sampling",
    "fab fa-creative-commons-sampling-plus",
    "fab fa-creative-commons-share",
    "fab fa-creative-commons-zero",
    "fas fa-credit-card",
    "far fa-credit-card",
    "fab fa-critical-role",
    "fas fa-crop",
    "fas fa-crop-alt",
    "fas fa-cross",
    "fas fa-crosshairs",
    "fas fa-crow",
    "fas fa-crown",
    "fas fa-crutch",
    "fab fa-css3",
    "fab fa-css3-alt",
    "fas fa-cube",
    "fas fa-cubes",
    "fas fa-cut",
    "fab fa-cuttlefish",
    "fab fa-d-and-d",
    "fab fa-d-and-d-beyond",
    "fab fa-dashcube",
    "fas fa-database",
    "fas fa-deaf",
    "fab fa-delicious",
    "fas fa-democrat",
    "fab fa-deploydog",
    "fab fa-deskpro",
    "fas fa-desktop",
    "fab fa-dev",
    "fab fa-deviantart",
    "fas fa-dharmachakra",
    "fab fa-dhl",
    "fas fa-diagnoses",
    "fab fa-diaspora",
    "fas fa-dice",
    "fas fa-dice-d20",
    "fas fa-dice-d6",
    "fas fa-dice-five",
    "fas fa-dice-four",
    "fas fa-dice-one",
    "fas fa-dice-six",
    "fas fa-dice-three",
    "fas fa-dice-two",
    "fab fa-digg",
    "fab fa-digital-ocean",
    "fas fa-digital-tachograph",
    "fas fa-directions",
    "fab fa-discord",
    "fab fa-discourse",
    "fas fa-divide",
    "fas fa-dizzy",
    "far fa-dizzy",
    "fas fa-dna",
    "fab fa-dochub",
    "fab fa-docker",
    "fas fa-dog",
    "fas fa-dollar-sign",
    "fas fa-dolly",
    "fas fa-dolly-flatbed",
    "fas fa-donate",
    "fas fa-door-closed",
    "fas fa-door-open",
    "fas fa-dot-circle",
    "far fa-dot-circle",
    "fas fa-dove",
    "fas fa-download",
    "fab fa-draft2digital",
    "fas fa-drafting-compass",
    "fas fa-dragon",
    "fas fa-draw-polygon",
    "fab fa-dribbble",
    "fab fa-dribbble-square",
    "fab fa-dropbox",
    "fas fa-drum",
    "fas fa-drum-steelpan",
    "fas fa-drumstick-bite",
    "fab fa-drupal",
    "fas fa-dumbbell",
    "fas fa-dumpster",
    "fas fa-dumpster-fire",
    "fas fa-dungeon",
    "fab fa-dyalog",
    "fab fa-earlybirds",
    "fab fa-ebay",
    "fab fa-edge",
    "fas fa-edit",
    "far fa-edit",
    "fas fa-egg",
    "fas fa-eject",
    "fab fa-elementor",
    "fas fa-ellipsis-h",
    "fas fa-ellipsis-v",
    "fab fa-ello",
    "fab fa-ember",
    "fab fa-empire",
    "fas fa-envelope",
    "far fa-envelope",
    "fas fa-envelope-open",
    "far fa-envelope-open",
    "fas fa-envelope-open-text",
    "fas fa-envelope-square",
    "fab fa-envira",
    "fas fa-equals",
    "fas fa-eraser",
    "fab fa-erlang",
    "fab fa-ethereum",
    "fas fa-ethernet",
    "fab fa-etsy",
    "fas fa-euro-sign",
    "fab fa-evernote",
    "fas fa-exchange-alt",
    "fas fa-exclamation",
    "fas fa-exclamation-circle",
    "fas fa-exclamation-triangle",
    "fas fa-expand",
    "fas fa-expand-arrows-alt",
    "fab fa-expeditedssl",
    "fas fa-external-link-alt",
    "fas fa-external-link-square-alt",
    "fas fa-eye",
    "far fa-eye",
    "fas fa-eye-dropper",
    "fas fa-eye-slash",
    "far fa-eye-slash",
    "fab fa-facebook",
    "fab fa-facebook-f",
    "fab fa-facebook-messenger",
    "fab fa-facebook-square",
    "fas fa-fan",
    "fab fa-fantasy-flight-games",
    "fas fa-fast-backward",
    "fas fa-fast-forward",
    "fas fa-fax",
    "fas fa-feather",
    "fas fa-feather-alt",
    "fab fa-fedex",
    "fab fa-fedora",
    "fas fa-female",
    "fas fa-fighter-jet",
    "fab fa-figma",
    "fas fa-file",
    "far fa-file",
    "fas fa-file-alt",
    "far fa-file-alt",
    "fas fa-file-archive",
    "far fa-file-archive",
    "fas fa-file-audio",
    "far fa-file-audio",
    "fas fa-file-code",
    "far fa-file-code",
    "fas fa-file-contract",
    "fas fa-file-csv",
    "fas fa-file-download",
    "fas fa-file-excel",
    "far fa-file-excel",
    "fas fa-file-export",
    "fas fa-file-image",
    "far fa-file-image",
    "fas fa-file-import",
    "fas fa-file-invoice",
    "fas fa-file-invoice-dollar",
    "fas fa-file-medical",
    "fas fa-file-medical-alt",
    "fas fa-file-pdf",
    "far fa-file-pdf",
    "fas fa-file-powerpoint",
    "far fa-file-powerpoint",
    "fas fa-file-prescription",
    "fas fa-file-signature",
    "fas fa-file-upload",
    "fas fa-file-video",
    "far fa-file-video",
    "fas fa-file-word",
    "far fa-file-word",
    "fas fa-fill",
    "fas fa-fill-drip",
    "fas fa-film",
    "fas fa-filter",
    "fas fa-fingerprint",
    "fas fa-fire",
    "fas fa-fire-alt",
    "fas fa-fire-extinguisher",
    "fab fa-firefox",
    "fas fa-first-aid",
    "fab fa-first-order",
    "fab fa-first-order-alt",
    "fab fa-firstdraft",
    "fas fa-fish",
    "fas fa-fist-raised",
    "fas fa-flag",
    "far fa-flag",
    "fas fa-flag-checkered",
    "fas fa-flag-usa",
    "fas fa-flask",
    "fab fa-flickr",
    "fab fa-flipboard",
    "fas fa-flushed",
    "far fa-flushed",
    "fab fa-fly",
    "fas fa-folder",
    "far fa-folder",
    "fas fa-folder-minus",
    "fas fa-folder-open",
    "far fa-folder-open",
    "fas fa-folder-plus",
    "fas fa-font",
    "fab fa-font-awesome",
    "fab fa-font-awesome-alt",
    "fab fa-font-awesome-flag",
    "fab fa-fonticons",
    "fab fa-fonticons-fi",
    "fas fa-football-ball",
    "fab fa-fort-awesome",
    "fab fa-fort-awesome-alt",
    "fab fa-forumbee",
    "fas fa-forward",
    "fab fa-foursquare",
    "fab fa-free-code-camp",
    "fab fa-freebsd",
    "fas fa-frog",
    "fas fa-frown",
    "far fa-frown",
    "fas fa-frown-open",
    "far fa-frown-open",
    "fab fa-fulcrum",
    "fas fa-funnel-dollar",
    "fas fa-futbol",
    "far fa-futbol",
    "fab fa-galactic-republic",
    "fab fa-galactic-senate",
    "fas fa-gamepad",
    "fas fa-gas-pump",
    "fas fa-gavel",
    "fas fa-gem",
    "far fa-gem",
    "fas fa-genderless",
    "fab fa-get-pocket",
    "fab fa-gg",
    "fab fa-gg-circle",
    "fas fa-ghost",
    "fas fa-gift",
    "fas fa-gifts",
    "fab fa-git",
    "fab fa-git-alt",
    "fab fa-git-square",
    "fab fa-github",
    "fab fa-github-alt",
    "fab fa-github-square",
    "fab fa-gitkraken",
    "fab fa-gitlab",
    "fab fa-gitter",
    "fas fa-glass-cheers",
    "fas fa-glass-martini",
    "fas fa-glass-martini-alt",
    "fas fa-glass-whiskey",
    "fas fa-glasses",
    "fab fa-glide",
    "fab fa-glide-g",
    "fas fa-globe",
    "fas fa-globe-africa",
    "fas fa-globe-americas",
    "fas fa-globe-asia",
    "fas fa-globe-europe",
    "fab fa-gofore",
    "fas fa-golf-ball",
    "fab fa-goodreads",
    "fab fa-goodreads-g",
    "fab fa-google",
    "fab fa-google-drive",
    "fab fa-google-play",
    "fab fa-google-plus",
    "fab fa-google-plus-g",
    "fab fa-google-plus-square",
    "fab fa-google-wallet",
    "fas fa-gopuram",
    "fas fa-graduation-cap",
    "fab fa-gratipay",
    "fab fa-grav",
    "fas fa-greater-than",
    "fas fa-greater-than-equal",
    "fas fa-grimace",
    "far fa-grimace",
    "fas fa-grin",
    "far fa-grin",
    "fas fa-grin-alt",
    "far fa-grin-alt",
    "fas fa-grin-beam",
    "far fa-grin-beam",
    "fas fa-grin-beam-sweat",
    "far fa-grin-beam-sweat",
    "fas fa-grin-hearts",
    "far fa-grin-hearts",
    "fas fa-grin-squint",
    "far fa-grin-squint",
    "fas fa-grin-squint-tears",
    "far fa-grin-squint-tears",
    "fas fa-grin-stars",
    "far fa-grin-stars",
    "fas fa-grin-tears",
    "far fa-grin-tears",
    "fas fa-grin-tongue",
    "far fa-grin-tongue",
    "fas fa-grin-tongue-squint",
    "far fa-grin-tongue-squint",
    "fas fa-grin-tongue-wink",
    "far fa-grin-tongue-wink",
    "fas fa-grin-wink",
    "far fa-grin-wink",
    "fas fa-grip-horizontal",
    "fas fa-grip-lines",
    "fas fa-grip-lines-vertical",
    "fas fa-grip-vertical",
    "fab fa-gripfire",
    "fab fa-grunt",
    "fas fa-guitar",
    "fab fa-gulp",
    "fas fa-h-square",
    "fab fa-hacker-news",
    "fab fa-hacker-news-square",
    "fab fa-hackerrank",
    "fas fa-hamburger",
    "fas fa-hammer",
    "fas fa-hamsa",
    "fas fa-hand-holding",
    "fas fa-hand-holding-heart",
    "fas fa-hand-holding-usd",
    "fas fa-hand-lizard",
    "far fa-hand-lizard",
    "fas fa-hand-middle-finger",
    "fas fa-hand-paper",
    "far fa-hand-paper",
    "fas fa-hand-peace",
    "far fa-hand-peace",
    "fas fa-hand-point-down",
    "far fa-hand-point-down",
    "fas fa-hand-point-left",
    "far fa-hand-point-left",
    "fas fa-hand-point-right",
    "far fa-hand-point-right",
    "fas fa-hand-point-up",
    "far fa-hand-point-up",
    "fas fa-hand-pointer",
    "far fa-hand-pointer",
    "fas fa-hand-rock",
    "far fa-hand-rock",
    "fas fa-hand-scissors",
    "far fa-hand-scissors",
    "fas fa-hand-spock",
    "far fa-hand-spock",
    "fas fa-hands",
    "fas fa-hands-helping",
    "fas fa-handshake",
    "far fa-handshake",
    "fas fa-hanukiah",
    "fas fa-hard-hat",
    "fas fa-hashtag",
    "fas fa-hat-cowboy",
    "fas fa-hat-cowboy-side",
    "fas fa-hat-wizard",
    "fas fa-haykal",
    "fas fa-hdd",
    "far fa-hdd",
    "fas fa-heading",
    "fas fa-headphones",
    "fas fa-headphones-alt",
    "fas fa-headset",
    "fas fa-heart",
    "far fa-heart",
    "fas fa-heart-broken",
    "fas fa-heartbeat",
    "fas fa-helicopter",
    "fas fa-highlighter",
    "fas fa-hiking",
    "fas fa-hippo",
    "fab fa-hips",
    "fab fa-hire-a-helper",
    "fas fa-history",
    "fas fa-hockey-puck",
    "fas fa-holly-berry",
    "fas fa-home",
    "fab fa-hooli",
    "fab fa-hornbill",
    "fas fa-horse",
    "fas fa-horse-head",
    "fas fa-hospital",
    "far fa-hospital",
    "fas fa-hospital-alt",
    "fas fa-hospital-symbol",
    "fas fa-hot-tub",
    "fas fa-hotdog",
    "fas fa-hotel",
    "fab fa-hotjar",
    "fas fa-hourglass",
    "far fa-hourglass",
    "fas fa-hourglass-end",
    "fas fa-hourglass-half",
    "fas fa-hourglass-start",
    "fas fa-house-damage",
    "fab fa-houzz",
    "fas fa-hryvnia",
    "fab fa-html5",
    "fab fa-hubspot",
    "fas fa-i-cursor",
    "fas fa-ice-cream",
    "fas fa-icicles",
    "fas fa-icons",
    "fas fa-id-badge",
    "far fa-id-badge",
    "fas fa-id-card",
    "far fa-id-card",
    "fas fa-id-card-alt",
    "fas fa-igloo",
    "fas fa-image",
    "far fa-image",
    "fas fa-images",
    "far fa-images",
    "fab fa-imdb",
    "fas fa-inbox",
    "fas fa-indent",
    "fas fa-industry",
    "fas fa-infinity",
    "fas fa-info",
    "fas fa-info-circle",
    "fab fa-instagram",
    "fab fa-intercom",
    "fab fa-internet-explorer",
    "fab fa-invision",
    "fab fa-ioxhost",
    "fas fa-italic",
    "fab fa-itch-io",
    "fab fa-itunes",
    "fab fa-itunes-note",
    "fab fa-java",
    "fas fa-jedi",
    "fab fa-jedi-order",
    "fab fa-jenkins",
    "fab fa-jira",
    "fab fa-joget",
    "fas fa-joint",
    "fab fa-joomla",
    "fas fa-journal-whills",
    "fab fa-js",
    "fab fa-js-square",
    "fab fa-jsfiddle",
    "fas fa-kaaba",
    "fab fa-kaggle",
    "fas fa-key",
    "fab fa-keybase",
    "fas fa-keyboard",
    "far fa-keyboard",
    "fab fa-keycdn",
    "fas fa-khanda",
    "fab fa-kickstarter",
    "fab fa-kickstarter-k",
    "fas fa-kiss",
    "far fa-kiss",
    "fas fa-kiss-beam",
    "far fa-kiss-beam",
    "fas fa-kiss-wink-heart",
    "far fa-kiss-wink-heart",
    "fas fa-kiwi-bird",
    "fab fa-korvue",
    "fas fa-landmark",
    "fas fa-language",
    "fas fa-laptop",
    "fas fa-laptop-code",
    "fas fa-laptop-medical",
    "fab fa-laravel",
    "fab fa-lastfm",
    "fab fa-lastfm-square",
    "fas fa-laugh",
    "far fa-laugh",
    "fas fa-laugh-beam",
    "far fa-laugh-beam",
    "fas fa-laugh-squint",
    "far fa-laugh-squint",
    "fas fa-laugh-wink",
    "far fa-laugh-wink",
    "fas fa-layer-group",
    "fas fa-leaf",
    "fab fa-leanpub",
    "fas fa-lemon",
    "far fa-lemon",
    "fab fa-less",
    "fas fa-less-than",
    "fas fa-less-than-equal",
    "fas fa-level-down-alt",
    "fas fa-level-up-alt",
    "fas fa-life-ring",
    "far fa-life-ring",
    "fas fa-lightbulb",
    "far fa-lightbulb",
    "fab fa-line",
    "fas fa-link",
    "fab fa-linkedin",
    "fab fa-linkedin-in",
    "fab fa-linode",
    "fab fa-linux",
    "fas fa-lira-sign",
    "fas fa-list",
    "fas fa-list-alt",
    "far fa-list-alt",
    "fas fa-list-ol",
    "fas fa-list-ul",
    "fas fa-location-arrow",
    "fas fa-lock",
    "fas fa-lock-open",
    "fas fa-long-arrow-alt-down",
    "fas fa-long-arrow-alt-left",
    "fas fa-long-arrow-alt-right",
    "fas fa-long-arrow-alt-up",
    "fas fa-low-vision",
    "fas fa-luggage-cart",
    "fab fa-lyft",
    "fab fa-magento",
    "fas fa-magic",
    "fas fa-magnet",
    "fas fa-mail-bulk",
    "fab fa-mailchimp",
    "fas fa-male",
    "fab fa-mandalorian",
    "fas fa-map",
    "far fa-map",
    "fas fa-map-marked",
    "fas fa-map-marked-alt",
    "fas fa-map-marker",
    "fas fa-map-marker-alt",
    "fas fa-map-pin",
    "fas fa-map-signs",
    "fab fa-markdown",
    "fas fa-marker",
    "fas fa-mars",
    "fas fa-mars-double",
    "fas fa-mars-stroke",
    "fas fa-mars-stroke-h",
    "fas fa-mars-stroke-v",
    "fas fa-mask",
    "fab fa-mastodon",
    "fab fa-maxcdn",
    "fab fa-mdb",
    "fas fa-medal",
    "fab fa-medapps",
    "fab fa-medium",
    "fab fa-medium-m",
    "fas fa-medkit",
    "fab fa-medrt",
    "fab fa-meetup",
    "fab fa-megaport",
    "fas fa-meh",
    "far fa-meh",
    "fas fa-meh-blank",
    "far fa-meh-blank",
    "fas fa-meh-rolling-eyes",
    "far fa-meh-rolling-eyes",
    "fas fa-memory",
    "fab fa-mendeley",
    "fas fa-menorah",
    "fas fa-mercury",
    "fas fa-meteor",
    "fas fa-microchip",
    "fas fa-microphone",
    "fas fa-microphone-alt",
    "fas fa-microphone-alt-slash",
    "fas fa-microphone-slash",
    "fas fa-microscope",
    "fab fa-microsoft",
    "fas fa-minus",
    "fas fa-minus-circle",
    "fas fa-minus-square",
    "far fa-minus-square",
    "fas fa-mitten",
    "fab fa-mix",
    "fab fa-mixcloud",
    "fab fa-mizuni",
    "fas fa-mobile",
    "fas fa-mobile-alt",
    "fab fa-modx",
    "fab fa-monero",
    "fas fa-money-bill",
    "fas fa-money-bill-alt",
    "far fa-money-bill-alt",
    "fas fa-money-bill-wave",
    "fas fa-money-bill-wave-alt",
    "fas fa-money-check",
    "fas fa-money-check-alt",
    "fas fa-monument",
    "fas fa-moon",
    "far fa-moon",
    "fas fa-mortar-pestle",
    "fas fa-mosque",
    "fas fa-motorcycle",
    "fas fa-mountain",
    "fas fa-mouse",
    "fas fa-mouse-pointer",
    "fas fa-mug-hot",
    "fas fa-music",
    "fab fa-napster",
    "fab fa-neos",
    "fas fa-network-wired",
    "fas fa-neuter",
    "fas fa-newspaper",
    "far fa-newspaper",
    "fab fa-nimblr",
    "fab fa-node",
    "fab fa-node-js",
    "fas fa-not-equal",
    "fas fa-notes-medical",
    "fab fa-npm",
    "fab fa-ns8",
    "fab fa-nutritionix",
    "fas fa-object-group",
    "far fa-object-group",
    "fas fa-object-ungroup",
    "far fa-object-ungroup",
    "fab fa-odnoklassniki",
    "fab fa-odnoklassniki-square",
    "fas fa-oil-can",
    "fab fa-old-republic",
    "fas fa-om",
    "fab fa-opencart",
    "fab fa-openid",
    "fab fa-opera",
    "fab fa-optin-monster",
    "fab fa-orcid",
    "fab fa-osi",
    "fas fa-otter",
    "fas fa-outdent",
    "fab fa-page4",
    "fab fa-pagelines",
    "fas fa-pager",
    "fas fa-paint-brush",
    "fas fa-paint-roller",
    "fas fa-palette",
    "fab fa-palfed",
    "fas fa-pallet",
    "fas fa-paper-plane",
    "far fa-paper-plane",
    "fas fa-paperclip",
    "fas fa-parachute-box",
    "fas fa-paragraph",
    "fas fa-parking",
    "fas fa-passport",
    "fas fa-pastafarianism",
    "fas fa-paste",
    "fab fa-patreon",
    "fas fa-pause",
    "fas fa-pause-circle",
    "far fa-pause-circle",
    "fas fa-paw",
    "fab fa-paypal",
    "fas fa-peace",
    "fas fa-pen",
    "fas fa-pen-alt",
    "fas fa-pen-fancy",
    "fas fa-pen-nib",
    "fas fa-pen-square",
    "fas fa-pencil-alt",
    "fas fa-pencil-ruler",
    "fab fa-penny-arcade",
    "fas fa-people-carry",
    "fas fa-pepper-hot",
    "fas fa-percent",
    "fas fa-percentage",
    "fab fa-periscope",
    "fas fa-person-booth",
    "fab fa-phabricator",
    "fab fa-phoenix-framework",
    "fab fa-phoenix-squadron",
    "fas fa-phone",
    "fas fa-phone-alt",
    "fas fa-phone-slash",
    "fas fa-phone-square",
    "fas fa-phone-square-alt",
    "fas fa-phone-volume",
    "fas fa-photo-video",
    "fab fa-php",
    "fab fa-pied-piper",
    "fab fa-pied-piper-alt",
    "fab fa-pied-piper-hat",
    "fab fa-pied-piper-pp",
    "fas fa-piggy-bank",
    "fas fa-pills",
    "fab fa-pinterest",
    "fab fa-pinterest-p",
    "fab fa-pinterest-square",
    "fas fa-pizza-slice",
    "fas fa-place-of-worship",
    "fas fa-plane",
    "fas fa-plane-arrival",
    "fas fa-plane-departure",
    "fas fa-play",
    "fas fa-play-circle",
    "far fa-play-circle",
    "fab fa-playstation",
    "fas fa-plug",
    "fas fa-plus",
    "fas fa-plus-circle",
    "fas fa-plus-square",
    "far fa-plus-square",
    "fas fa-podcast",
    "fas fa-poll",
    "fas fa-poll-h",
    "fas fa-poo",
    "fas fa-poo-storm",
    "fas fa-poop",
    "fas fa-portrait",
    "fas fa-pound-sign",
    "fas fa-power-off",
    "fas fa-pray",
    "fas fa-praying-hands",
    "fas fa-prescription",
    "fas fa-prescription-bottle",
    "fas fa-prescription-bottle-alt",
    "fas fa-print",
    "fas fa-procedures",
    "fab fa-product-hunt",
    "fas fa-project-diagram",
    "fab fa-pushed",
    "fas fa-puzzle-piece",
    "fab fa-python",
    "fab fa-qq",
    "fas fa-qrcode",
    "fas fa-question",
    "fas fa-question-circle",
    "far fa-question-circle",
    "fas fa-quidditch",
    "fab fa-quinscape",
    "fab fa-quora",
    "fas fa-quote-left",
    "fas fa-quote-right",
    "fas fa-quran",
    "fab fa-r-project",
    "fas fa-radiation",
    "fas fa-radiation-alt",
    "fas fa-rainbow",
    "fas fa-random",
    "fab fa-raspberry-pi",
    "fab fa-ravelry",
    "fab fa-react",
    "fab fa-reacteurope",
    "fab fa-readme",
    "fab fa-rebel",
    "fas fa-receipt",
    "fas fa-record-vinyl",
    "fas fa-recycle",
    "fab fa-red-river",
    "fab fa-reddit",
    "fab fa-reddit-alien",
    "fab fa-reddit-square",
    "fab fa-redhat",
    "fas fa-redo",
    "fas fa-redo-alt",
    "fas fa-registered",
    "far fa-registered",
    "fas fa-remove-format",
    "fab fa-renren",
    "fas fa-reply",
    "fas fa-reply-all",
    "fab fa-replyd",
    "fas fa-republican",
    "fab fa-researchgate",
    "fab fa-resolving",
    "fas fa-restroom",
    "fas fa-retweet",
    "fab fa-rev",
    "fas fa-ribbon",
    "fas fa-ring",
    "fas fa-road",
    "fas fa-robot",
    "fas fa-rocket",
    "fab fa-rocketchat",
    "fab fa-rockrms",
    "fas fa-route",
    "fas fa-rss",
    "fas fa-rss-square",
    "fas fa-ruble-sign",
    "fas fa-ruler",
    "fas fa-ruler-combined",
    "fas fa-ruler-horizontal",
    "fas fa-ruler-vertical",
    "fas fa-running",
    "fas fa-rupee-sign",
    "fas fa-sad-cry",
    "far fa-sad-cry",
    "fas fa-sad-tear",
    "far fa-sad-tear",
    "fab fa-safari",
    "fab fa-salesforce",
    "fab fa-sass",
    "fas fa-satellite",
    "fas fa-satellite-dish",
    "fas fa-save",
    "far fa-save",
    "fab fa-schlix",
    "fas fa-school",
    "fas fa-screwdriver",
    "fab fa-scribd",
    "fas fa-scroll",
    "fas fa-sd-card",
    "fas fa-search",
    "fas fa-search-dollar",
    "fas fa-search-location",
    "fas fa-search-minus",
    "fas fa-search-plus",
    "fab fa-searchengin",
    "fas fa-seedling",
    "fab fa-sellcast",
    "fab fa-sellsy",
    "fas fa-server",
    "fab fa-servicestack",
    "fas fa-shapes",
    "fas fa-share",
    "fas fa-share-alt",
    "fas fa-share-alt-square",
    "fas fa-share-square",
    "far fa-share-square",
    "fas fa-shekel-sign",
    "fas fa-shield-alt",
    "fas fa-ship",
    "fas fa-shipping-fast",
    "fab fa-shirtsinbulk",
    "fas fa-shoe-prints",
    "fas fa-shopping-bag",
    "fas fa-shopping-basket",
    "fas fa-shopping-cart",
    "fab fa-shopware",
    "fas fa-shower",
    "fas fa-shuttle-van",
    "fas fa-sign",
    "fas fa-sign-in-alt",
    "fas fa-sign-language",
    "fas fa-sign-out-alt",
    "fas fa-signal",
    "fas fa-signature",
    "fas fa-sim-card",
    "fab fa-simplybuilt",
    "fab fa-sistrix",
    "fas fa-sitemap",
    "fab fa-sith",
    "fas fa-skating",
    "fab fa-sketch",
    "fas fa-skiing",
    "fas fa-skiing-nordic",
    "fas fa-skull",
    "fas fa-skull-crossbones",
    "fab fa-skyatlas",
    "fab fa-skype",
    "fab fa-slack",
    "fab fa-slack-hash",
    "fas fa-slash",
    "fas fa-sleigh",
    "fas fa-sliders-h",
    "fab fa-slideshare",
    "fas fa-smile",
    "far fa-smile",
    "fas fa-smile-beam",
    "far fa-smile-beam",
    "fas fa-smile-wink",
    "far fa-smile-wink",
    "fas fa-smog",
    "fas fa-smoking",
    "fas fa-smoking-ban",
    "fas fa-sms",
    "fab fa-snapchat",
    "fab fa-snapchat-ghost",
    "fab fa-snapchat-square",
    "fas fa-snowboarding",
    "fas fa-snowflake",
    "far fa-snowflake",
    "fas fa-snowman",
    "fas fa-snowplow",
    "fas fa-socks",
    "fas fa-solar-panel",
    "fas fa-sort",
    "fas fa-sort-alpha-down",
    "fas fa-sort-alpha-down-alt",
    "fas fa-sort-alpha-up",
    "fas fa-sort-alpha-up-alt",
    "fas fa-sort-amount-down",
    "fas fa-sort-amount-down-alt",
    "fas fa-sort-amount-up",
    "fas fa-sort-amount-up-alt",
    "fas fa-sort-down",
    "fas fa-sort-numeric-down",
    "fas fa-sort-numeric-down-alt",
    "fas fa-sort-numeric-up",
    "fas fa-sort-numeric-up-alt",
    "fas fa-sort-up",
    "fab fa-soundcloud",
    "fab fa-sourcetree",
    "fas fa-spa",
    "fas fa-space-shuttle",
    "fab fa-speakap",
    "fab fa-speaker-deck",
    "fas fa-spell-check",
    "fas fa-spider",
    "fas fa-spinner",
    "fas fa-splotch",
    "fab fa-spotify",
    "fas fa-spray-can",
    "fas fa-square",
    "far fa-square",
    "fas fa-square-full",
    "fas fa-square-root-alt",
    "fab fa-squarespace",
    "fab fa-stack-exchange",
    "fab fa-stack-overflow",
    "fab fa-stackpath",
    "fas fa-stamp",
    "fas fa-star",
    "far fa-star",
    "fas fa-star-and-crescent",
    "fas fa-star-half",
    "far fa-star-half",
    "fas fa-star-half-alt",
    "fas fa-star-of-david",
    "fas fa-star-of-life",
    "fab fa-staylinked",
    "fab fa-steam",
    "fab fa-steam-square",
    "fab fa-steam-symbol",
    "fas fa-step-backward",
    "fas fa-step-forward",
    "fas fa-stethoscope",
    "fab fa-sticker-mule",
    "fas fa-sticky-note",
    "far fa-sticky-note",
    "fas fa-stop",
    "fas fa-stop-circle",
    "far fa-stop-circle",
    "fas fa-stopwatch",
    "fas fa-store",
    "fas fa-store-alt",
    "fab fa-strava",
    "fas fa-stream",
    "fas fa-street-view",
    "fas fa-strikethrough",
    "fab fa-stripe",
    "fab fa-stripe-s",
    "fas fa-stroopwafel",
    "fab fa-studiovinari",
    "fab fa-stumbleupon",
    "fab fa-stumbleupon-circle",
    "fas fa-subscript",
    "fas fa-subway",
    "fas fa-suitcase",
    "fas fa-suitcase-rolling",
    "fas fa-sun",
    "far fa-sun",
    "fab fa-superpowers",
    "fas fa-superscript",
    "fab fa-supple",
    "fas fa-surprise",
    "far fa-surprise",
    "fab fa-suse",
    "fas fa-swatchbook",
    "fab fa-swift",
    "fas fa-swimmer",
    "fas fa-swimming-pool",
    "fab fa-symfony",
    "fas fa-synagogue",
    "fas fa-sync",
    "fas fa-sync-alt",
    "fas fa-syringe",
    "fas fa-table",
    "fas fa-table-tennis",
    "fas fa-tablet",
    "fas fa-tablet-alt",
    "fas fa-tablets",
    "fas fa-tachometer-alt",
    "fas fa-tag",
    "fas fa-tags",
    "fas fa-tape",
    "fas fa-tasks",
    "fas fa-taxi",
    "fab fa-teamspeak",
    "fas fa-teeth",
    "fas fa-teeth-open",
    "fab fa-telegram",
    "fab fa-telegram-plane",
    "fas fa-temperature-high",
    "fas fa-temperature-low",
    "fab fa-tencent-weibo",
    "fas fa-tenge",
    "fas fa-terminal",
    "fas fa-text-height",
    "fas fa-text-width",
    "fas fa-th",
    "fas fa-th-large",
    "fas fa-th-list",
    "fab fa-the-red-yeti",
    "fas fa-theater-masks",
    "fab fa-themeco",
    "fab fa-themeisle",
    "fas fa-thermometer",
    "fas fa-thermometer-empty",
    "fas fa-thermometer-full",
    "fas fa-thermometer-half",
    "fas fa-thermometer-quarter",
    "fas fa-thermometer-three-quarters",
    "fab fa-think-peaks",
    "fas fa-thumbs-down",
    "far fa-thumbs-down",
    "fas fa-thumbs-up",
    "far fa-thumbs-up",
    "fas fa-thumbtack",
    "fas fa-ticket-alt",
    "fas fa-times",
    "fas fa-times-circle",
    "far fa-times-circle",
    "fas fa-tint",
    "fas fa-tint-slash",
    "fas fa-tired",
    "far fa-tired",
    "fas fa-toggle-off",
    "fas fa-toggle-on",
    "fas fa-toilet",
    "fas fa-toilet-paper",
    "fas fa-toolbox",
    "fas fa-tools",
    "fas fa-tooth",
    "fas fa-torah",
    "fas fa-torii-gate",
    "fas fa-tractor",
    "fab fa-trade-federation",
    "fas fa-trademark",
    "fas fa-traffic-light",
    "fas fa-train",
    "fas fa-tram",
    "fas fa-transgender",
    "fas fa-transgender-alt",
    "fas fa-trash",
    "fas fa-trash-alt",
    "far fa-trash-alt",
    "fas fa-trash-restore",
    "fas fa-trash-restore-alt",
    "fas fa-tree",
    "fab fa-trello",
    "fab fa-tripadvisor",
    "fas fa-trophy",
    "fas fa-truck",
    "fas fa-truck-loading",
    "fas fa-truck-monster",
    "fas fa-truck-moving",
    "fas fa-truck-pickup",
    "fas fa-tshirt",
    "fas fa-tty",
    "fab fa-tumblr",
    "fab fa-tumblr-square",
    "fas fa-tv",
    "fab fa-twitch",
    "fab fa-twitter",
    "fab fa-twitter-square",
    "fab fa-typo3",
    "fab fa-uber",
    "fab fa-ubuntu",
    "fab fa-uikit",
    "fab fa-umbraco",
    "fas fa-umbrella",
    "fas fa-umbrella-beach",
    "fas fa-underline",
    "fas fa-undo",
    "fas fa-undo-alt",
    "fab fa-uniregistry",
    "fas fa-universal-access",
    "fas fa-university",
    "fas fa-unlink",
    "fas fa-unlock",
    "fas fa-unlock-alt",
    "fab fa-untappd",
    "fas fa-upload",
    "fab fa-ups",
    "fab fa-usb",
    "fas fa-user",
    "far fa-user",
    "fas fa-user-alt",
    "fas fa-user-alt-slash",
    "fas fa-user-astronaut",
    "fas fa-user-check",
    "fas fa-user-circle",
    "far fa-user-circle",
    "fas fa-user-clock",
    "fas fa-user-cog",
    "fas fa-user-edit",
    "fas fa-user-friends",
    "fas fa-user-graduate",
    "fas fa-user-injured",
    "fas fa-user-lock",
    "fas fa-user-md",
    "fas fa-user-minus",
    "fas fa-user-ninja",
    "fas fa-user-nurse",
    "fas fa-user-plus",
    "fas fa-user-secret",
    "fas fa-user-shield",
    "fas fa-user-slash",
    "fas fa-user-tag",
    "fas fa-user-tie",
    "fas fa-user-times",
    "fas fa-users",
    "fas fa-users-cog",
    "fab fa-usps",
    "fab fa-ussunnah",
    "fas fa-utensil-spoon",
    "fas fa-utensils",
    "fab fa-vaadin",
    "fas fa-vector-square",
    "fas fa-venus",
    "fas fa-venus-double",
    "fas fa-venus-mars",
    "fab fa-viacoin",
    "fab fa-viadeo",
    "fab fa-viadeo-square",
    "fas fa-vial",
    "fas fa-vials",
    "fab fa-viber",
    "fas fa-video",
    "fas fa-video-slash",
    "fas fa-vihara",
    "fab fa-vimeo",
    "fab fa-vimeo-square",
    "fab fa-vimeo-v",
    "fab fa-vine",
    "fab fa-vk",
    "fab fa-vnv",
    "fas fa-voicemail",
    "fas fa-volleyball-ball",
    "fas fa-volume-down",
    "fas fa-volume-mute",
    "fas fa-volume-off",
    "fas fa-volume-up",
    "fas fa-vote-yea",
    "fas fa-vr-cardboard",
    "fab fa-vuejs",
    "fas fa-walking",
    "fas fa-wallet",
    "fas fa-warehouse",
    "fas fa-water",
    "fas fa-wave-square",
    "fab fa-waze",
    "fab fa-weebly",
    "fab fa-weibo",
    "fas fa-weight",
    "fas fa-weight-hanging",
    "fab fa-weixin",
    "fab fa-whatsapp",
    "fab fa-whatsapp-square",
    "fas fa-wheelchair",
    "fab fa-whmcs",
    "fas fa-wifi",
    "fab fa-wikipedia-w",
    "fas fa-wind",
    "fas fa-window-close",
    "far fa-window-close",
    "fas fa-window-maximize",
    "far fa-window-maximize",
    "fas fa-window-minimize",
    "far fa-window-minimize",
    "fas fa-window-restore",
    "far fa-window-restore",
    "fab fa-windows",
    "fas fa-wine-bottle",
    "fas fa-wine-glass",
    "fas fa-wine-glass-alt",
    "fab fa-wix",
    "fab fa-wizards-of-the-coast",
    "fab fa-wolf-pack-battalion",
    "fas fa-won-sign",
    "fab fa-wordpress",
    "fab fa-wordpress-simple",
    "fab fa-wpbeginner",
    "fab fa-wpexplorer",
    "fab fa-wpforms",
    "fab fa-wpressr",
    "fas fa-wrench",
    "fas fa-x-ray",
    "fab fa-xbox",
    "fab fa-xing",
    "fab fa-xing-square",
    "fab fa-y-combinator",
    "fab fa-yahoo",
    "fab fa-yammer",
    "fab fa-yandex",
    "fab fa-yandex-international",
    "fab fa-yarn",
    "fab fa-yelp",
    "fas fa-yen-sign",
    "fas fa-yin-yang",
    "fab fa-yoast",
    "fab fa-youtube",
    "fab fa-youtube-square",
    "fab fa-zhihu"
]


class Index(gatehandler.Handler):
    def __init__(self, request):
        self.list_icons = list_icons
        self._cont_icon_list = 0
        self._max_icon_list = len(self.list_icons)
        gatehandler.Handler.__init__(self, request)

    def initialize(self):
        html = CONCATENATE(
            DIV(
                DIV(
                    DIV(
                        DIV("COMPONENTES", _class="phanterpwa-breadcrumb"),
                        DIV("FONT AWESOME", _class="phanterpwa-breadcrumb"),
                        _class="phanterpwa-breadcrumb-wrapper"
                    ),
                    _class="p-container"),
                _class='title_page_container card'
            ),
            DIV(
                DIV(
                    DIV(
                        DIV(
                            widgets.Preloaders("preloader_fontawesome"),
                            _style="width:100%; text-align: center; padding-top: 100px; padding-bottom: 100px;"
                        ),
                        _id="icon_awesome_container", _class="icon_awesome_container e-padding_20"),
                    _class="card p-row padding_20"
                ),
                _class="phanterpwa-container p-container"
            )
        )
        html.html_to("#main-container")
        BackButton = left_bar.LeftBarButton(
            "back_localizar_aluno",
            "Voltar",
            I(_class="fas fa-arrow-circle-left"),
            **{"_phanterpwa-way": "home",
                "position": "top",
                "ways": ["fontawesome"]}
        )
        window.PhanterPWA.Components['left_bar'].add_button(BackButton)
        setTimeout(lambda: self._add_content(0), 300)

    def _sampled(self, value):
        html = DIV(
            DIV(
                DIV(
                    widgets.MenuBox(
                        value.replace(" ", "_"),
                        DIV(I(_class=value), _class="icon_button wave_on_click"),
                        custom_menu=DIV(
                            DIV(
                                DIV(
                                    LABEL("Python"),
                                    INPUT(_value="I(_class=\"{0}\")".format(value)),
                                    _class="label_and_input"
                                ),
                                DIV(
                                    I(_class="fas fa-clipboard"),
                                    **{
                                        "_data-src": "I(_class=\"{0}\")".format(value),
                                        "_class": "copy_buttom"
                                    }
                                ),
                                _class="icon_awesome_panel"
                            ),
                            DIV(
                                DIV(
                                    LABEL("Html"),
                                    INPUT(_value=str(I(_class=value))),
                                    _class="label_and_input"
                                ),
                                DIV(
                                    I(_class="fas fa-clipboard"),
                                    **{
                                        "_data-src": str(I(_class=value)),
                                        "_class": "copy_buttom"
                                    }
                                ),
                                _class="icon_awesome_panel"
                            ),
                            DIV(
                                DIV(
                                    LABEL("Class"),
                                    INPUT(_value=value),
                                    _class="label_and_input"
                                ),
                                DIV(
                                    I(_class="fas fa-clipboard"),
                                    **{
                                        "_data-src": value,
                                        "_class": "copy_buttom"
                                    }
                                ),
                                _class="icon_awesome_panel"
                            ),
                            _class="icon_awesome_panel_wrapper"
                        ),
                        onOpen=self._on_icon_open
                    ),
                    **{"_data-icon": value, "_class": "icon_awesome-button"}
                ),
                DIV(
                    TABLE(
                        TR(
                            TD(" ".join(value.split(" ")[1].split("-")[1:]))
                        )
                    ),
                    _class="icon_awesome-label"
                ),
                _class="icon_awesome-wrapper"
            ),
            _class="p-col w1p50 w3p25 w4p14",
        )
        return html

    def _on_icon_open(self, el):
        jQuery(el).find(".copy_buttom").off("click").on("click", lambda: self._copy(this))

    def _add_content(self, v):
        html = CONCATENATE()
        start = v
        stop = v + 20
        if stop > self._max_icon_list:
            stop = self._max_icon_list
        cont = 0
        for x in range(start, stop):
            h = self._sampled(self.list_icons[x])
            html.append(h)
            cont += 1
        else:
            if v == 0:
                html.html_to("#icon_awesome_container")
                self._async_add_content(cont)
            else:
                html.append_to("#icon_awesome_container")
                self._async_add_content(stop)

    def _async_add_content(self, cont):
        setTimeout(lambda: self._add_content(cont))

    def _copy(self, el):
        p = jQuery(el).parent()
        p.find("input").select()
        document.execCommand("copy")

    def reload(self):
        self.start()


    def _after_load(self, data):
        console.log(data)
        XML(data).html_to("#icon_awesome_container")





import os
import unittest
from phanterpwa.tools import (
    interpolate,
    sass_change_vars,
    sass_map_vars,
    config,
    text_normalize,
    string_escape,
    one_space,
    split_seconds,
    join_seconds,
    humanize_seconds
)
from phanterpwa.i18n import Translator

CURRENT_PATH = os.path.dirname(__file__)
my_translator = Translator(os.path.join(CURRENT_PATH, "test_tools_humanize_seconds"), "humanize_seconds")

SAMPLE_tools = """$STROKEWIDTH: 8px /*teste*/
$_BORDERRADIUS: 100%
$BORDERRADIUS: 100%
$border-dark: rgba($base-color, 0.88)
\t$SIM: #9px
$black: #000 !default
.phanterpwa-components-preloaders-android
  $STROKEWIDTH: 8px //teste
  $CONTAINERWIDTH: 60px
  // $NAO: 70
  /* $NAO2: 70 */
  *
    box-sizing: border-box
  .left
      float: left {{important}}
  .right
      float: right !important
  .preloader-wrapper
    display: inline-block
    position: relative
    width: {{no_var}}
    height: $CONTAINERWIDTH"""
SAMPLE_tools2 = """$STROKEWIDTH: 8px /*teste*/
$_BORDERRADIUS: 100%
$BORDERRADIUS: 100%
$border-dark: rgba($base-color, 0.88)
\t$SIM: #9px
$black: #000 !default
.phanterpwa-components-preloaders-android
  $STROKEWIDTH: 8px //teste
  $CONTAINERWIDTH: 60px
  // $NAO: 70
  /* $NAO2: 70 */
  *
    box-sizing: border-box
  .left
      float: left !important
  .right
      float: right !important
  .preloader-wrapper
    display: inline-block
    position: relative
    width: $CONTAINERWIDTH
    height: $CONTAINERWIDTH"""

SAMPLE_tools3 = """$STROKEWIDTH: 15px
$_BORDERRADIUS: 100%
$BORDERRADIUS: 100%
$border-dark: rgba($base-color, 0.88)
\t$SIM: #9px
$black: #000 !default
.phanterpwa-components-preloaders-android
  $STROKEWIDTH: 15px
  $CONTAINERWIDTH: 200px
  // $NAO: 70
  /* $NAO2: 70 */
  *
    box-sizing: border-box
  .left
      float: left !important
  .right
      float: right !important
  .preloader-wrapper
    display: inline-block
    position: relative
    width: $CONTAINERWIDTH
    height: $CONTAINERWIDTH"""


class TestTools(unittest.TestCase):
    def test01_interpolate(self):
        self.assertEqual(
            interpolate(SAMPLE_tools, {"no_var": "$CONTAINERWIDTH", "not_exist": "None", "important": "!important"}),
            SAMPLE_tools2
        )
        self.assertEqual(
            interpolate(SAMPLE_tools, {"no_var2": "$CONTAINERWIDTH", "not_exist2": "None", "important2": "!important"}),
            SAMPLE_tools
        )

    def test02_sass_map_vars(self):
        SAMPLE = {
            "STROKEWIDTH": "8px",
            "BORDERRADIUS": "100%",
            "_BORDERRADIUS": "100%",
            "border-dark": "rgba($base-color, 0.88)",
            "SIM": "#9px",
            "black": "#000 !default",
            "CONTAINERWIDTH": "60px"
        }
        res = sass_map_vars(SAMPLE_tools2)
        for x in res:
            self.assertEqual(
                SAMPLE[x],
                res[x]
            )

        self.assertEqual(
            sass_map_vars("ssdf dfdsd sdsdf\n sdfsdf"),
            {}
        )

    def test03_sass_change_vars(self):
        self.assertEqual(
            sass_change_vars(SAMPLE_tools2, {"STROKEWIDTH": "15px",
                "CONTAINERWIDTH": "200px", "NOT_EXIST": "600px"}),
            SAMPLE_tools3
        )
        self.assertEqual(
            sass_change_vars(SAMPLE_tools2, {"STROKEWIDTH2": "15px",
                "CONTAINERWIDTH2": "200px", "NOT_EXIST": "600px"}),
            SAMPLE_tools2
        )

    def test04_config(self):
        if os.path.exists(os.path.join(CURRENT_PATH, "subfolder", "config.json")):
            os.remove(os.path.join(CURRENT_PATH, "subfolder", "config.json"))
        if os.path.exists(os.path.join(CURRENT_PATH, "subfolder", "cfg.json")):
            os.remove(os.path.join(CURRENT_PATH, "subfolder", "cfg.json"))
        if os.path.exists(os.path.join(CURRENT_PATH, "test_folder")):
            import glob
            c = glob.glob(os.path.join(CURRENT_PATH, "test_folder", "*"))
            for x in c:
                os.remove(x)
            os.removedirs(os.path.join(CURRENT_PATH, "test_folder"))
        self.assertEqual(os.path.exists(os.path.join(CURRENT_PATH, "subfolder", "config.json")), False)
        self.assertEqual(os.path.exists(os.path.join(CURRENT_PATH, "subfolder", "cfg.json")), False)
        self.assertEqual(os.path.exists(os.path.join(CURRENT_PATH, "test_folder")), False)
        config(os.path.join(CURRENT_PATH, "test_folder"))
        self.assertEqual(os.path.exists(os.path.join(CURRENT_PATH, "test_folder.json")), True)
        os.remove(os.path.join(CURRENT_PATH, "test_folder.json"))
        self.assertEqual(os.path.exists(os.path.join(CURRENT_PATH, "test_folder.json")), False)
        config(os.path.join(CURRENT_PATH, "test_folder", "cfg.json"))
        self.assertEqual(os.path.exists(os.path.join(CURRENT_PATH, "test_folder", "cfg.json")), True)
        config(os.path.join(CURRENT_PATH, "test_folder", "cfg"))
        self.assertEqual(os.path.exists(os.path.join(CURRENT_PATH, "test_folder", "cfg.json")), True)
        config(os.path.join(CURRENT_PATH, "test_folder", "cfgij"))
        self.assertEqual(os.path.exists(os.path.join(CURRENT_PATH, "test_folder", "cfgij.json")), True)
        config(os.path.join(CURRENT_PATH, "test_folder"))
        self.assertEqual(os.path.exists(os.path.join(CURRENT_PATH, "test_folder", "config.json")), True)
        content = {"teste": "one"}
        change = {"teste": "two", "new": "one"}
        acumulate = {"teste": "two", "new": "one", "just": "three"}
        res = config(os.path.join(CURRENT_PATH, "test_folder"), content)
        self.assertEqual(res, content)
        res = config(os.path.join(CURRENT_PATH, "test_folder"), change)
        self.assertEqual(res, change)
        res = config(os.path.join(CURRENT_PATH, "test_folder"), {"just": "three"})
        self.assertEqual(res, acumulate)
        res = config(os.path.join(CURRENT_PATH, "test_folder"), {"just": "three"}, rewrite=True)
        self.assertEqual(res, {"just": "three"})

    def test05_text_normalize(self):
        self.assertEqual(
            text_normalize("Maçã"),
            "MACA"
        )
        self.assertEqual(
            text_normalize("Maçã", False),
            "Maca"
        )

    def test06_string_escape(self):
        self.assertEqual(
            string_escape("<div>the content</div>"),
            '&lt;div&gt;the content&lt;/div&gt;'
        )
        result = string_escape({
            "one": "<div style='test'>one</div>",
            "two": ["<div>two</div>"],
            "three": ("<div>three</div>",),
            "four": {"<div>four</div>"},
            "five": {
                "five_one": ["<br>", ("<input>", "<hr>"), {"<a href='link'></a>"}]
            }
        })
        self.assertEqual(
            result["one"],
            '&lt;div style=&#39;test&#39;&gt;one&lt;/div&gt;'
        )
        self.assertEqual(
            result["two"],
            ["&lt;div&gt;two&lt;/div&gt;"]
        )
        self.assertEqual(
            result["three"],
            ["&lt;div&gt;three&lt;/div&gt;"]
        )
        self.assertEqual(
            result["four"],
            ['&lt;div&gt;four&lt;/div&gt;']
        )
        self.assertEqual(
            result["five"]["five_one"],
            ["&lt;br&gt;", ["&lt;input&gt;", "&lt;hr&gt;"], ["&lt;a href=&#39;link&#39;&gt;&lt;/a&gt;"]]
        )

    def test07_one_space(self):
        self.assertEqual(
            one_space("   My    long \r\n text. \tTabulation,      spaces,     spaces.         "),
            "My long text. Tabulation, spaces, spaces."
        )

    def test08_split_seconds(self):
        self.assertEqual(
            split_seconds(123456789),
            {'year': 3, 'month': 11, 'day': 3, 'hour': 21, 'minute': 33, 'second': 9}
        )
        self.assertEqual(
            split_seconds(121),
            {'minute': 2, 'second': 1}
        )
        self.assertEqual(
            split_seconds(3659),
            {'hour': 1, 'second': 59}
        )

    def test09_join_seconds(self):
        self.assertEqual(
            join_seconds({'year': 1, 'hour': 1, 'second': 1}),
            31539601
        )
        self.assertEqual(
            join_seconds({'year': 1, 'second': 3601}),
            31539601
        )
        self.assertEqual(
            join_seconds({'year': 1, 'minute': 60, 'second': 1}),
            31539601
        )

    def test10_humanize_seconds(self):
        self.assertEqual(
            humanize_seconds(123456789),
            '3 years, 11 months, 3 days, 21 hours, 33 minutes and 9 seconds'
        )
        self.assertEqual(
            humanize_seconds(121),
            '2 minutes and 1 second'
        )
        {'minute': 2, 'second': 1}
        self.assertEqual(
            humanize_seconds(3659),
            '1 hour and 59 seconds'
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "year", "ano"),
            {"pt-BR": {"year": "ano"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "years", "anos"),
            {"pt-BR": {"years": "anos"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "month", "mês"),
            {"pt-BR": {"month": "mês"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "months", "meses"),
            {"pt-BR": {"months": "meses"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "day", "dia"),
            {"pt-BR": {"day": "dia"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "days", "dias"),
            {"pt-BR": {"days": "dias"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "hour", "hora"),
            {"pt-BR": {"hour": "hora"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "hours", "horas"),
            {"pt-BR": {"hours": "horas"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "minute", "minuto"),
            {"pt-BR": {"minute": "minuto"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "minutes", "minutos"),
            {"pt-BR": {"minutes": "minutos"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "second", "segundo"),
            {"pt-BR": {"second": "segundo"}}
        )
        self.assertEqual(
            my_translator.translate("pt-BR", "seconds", "segundos"),
            {"pt-BR": {"seconds": "segundos"}}
        )
        my_translator.direct_translation = "pt-BR"
        self.assertEqual(
            humanize_seconds(123456789, my_translator),
            '3 anos, 11 meses, 3 dias, 21 horas, 33 minutos and 9 segundos'
        )
        my_translator.direct_translation = "en-US"
        self.assertEqual(
            humanize_seconds(123456789, my_translator),
            '3 years, 11 months, 3 days, 21 hours, 33 minutes and 9 seconds'
        )
        self.assertEqual(
            humanize_seconds(3659, my_translator),
            '1 hour and 59 seconds'
        )
        my_translator.direct_translation = "pt-BR"
        self.assertEqual(
            humanize_seconds(3659, my_translator),
            '1 hora and 59 segundos'
        )
        my_translator.translate("pt-BR", "and ", "e ")
        self.assertEqual(
            humanize_seconds(3659, my_translator),
            '1 hora e 59 segundos'
        )

if __name__ == '__main__':
    unittest.main()



import os
import glob
files = glob.glob(os.path.join(os.path.dirname(__file__), "*.py"))
__all__ = [os.path.basename(x)[0:-3] for x in files if os.path.basename(x) != "__init__.py" and x[-3:] == ".py"]



import os
import sys
import logging
from phanterpwa.configer import ProjectConfig
from phanterpwa.i18n import Translator
_current_dir = os.path.dirname(__file__)
projectConfig = ProjectConfig(os.path.join(_current_dir, "config.json"))
_debug = projectConfig['PROJECT']['debug']
_project_name = projectConfig['PROJECT']['name']
# for app in projectConfig['FRONTEND'].keys():
#     Translator_apps = Translator(os.path.join(_current_dir, "frontapps", app, "languages"), identifier="{0}-{1}".format(
#         _project_name, app), debug=_debug)
#     Translator_apps.add_language("pt-BR")
# Translator_email = Translator(os.path.join(_current_dir, "api", "languages", "email"), identifier="{0}-email".format(
#     _project_name), debug=_debug)
# Translator_email.add_language("pt-BR")
# Translator_captcha = Translator(os.path.join(_current_dir, "api", "languages", "captcha"), identifier="{0}-captcha".format(
#     _project_name), debug=_debug)
# Translator_captcha.add_language("pt-BR")
# Translator_api = Translator(os.path.join(_current_dir, "api", "languages"), identifier="{0}-api".format(
#     _project_name), debug=_debug)
# Translator_api.add_language("pt-BR")
formatter = logging.Formatter(
    '%(asctime)s - %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S'
)

formatter_out_app = logging.Formatter(
    '%(asctime)s - %(name)s.app -  %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S'
)
fh_app = logging.FileHandler(os.path.join(_current_dir, 'logs', 'app.log'))
fh_app.setLevel(logging.ERROR)
fh_app.setFormatter(formatter)
fh_app_warnings = logging.FileHandler(os.path.join(_current_dir, 'logs', 'app.log'))
fh_app_warnings.setLevel(logging.WARNING)
fh_app_warnings.setFormatter(formatter)

sh_app_info = logging.StreamHandler(sys.stdout)
sh_app_info.setLevel(logging.INFO)
sh_app_info.setFormatter(formatter_out_app)

logger_app = logging.getLogger("{0}".format(os.path.basename(_current_dir)))
logger_app.addHandler(fh_app)
logger_app.addHandler(fh_app_warnings)
logger_app.addHandler(sh_app_info)


formatter_out_api = logging.Formatter(
    '%(asctime)s - %(name)s -  %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S'
)
fh_api = logging.FileHandler(os.path.join(_current_dir, 'logs', 'api.log'))
fh_api.setLevel(logging.ERROR)
fh_api.setFormatter(formatter)
fh_api_warnings = logging.FileHandler(os.path.join(_current_dir, 'logs', 'api.log'))
fh_api_warnings.setLevel(logging.WARNING)
fh_api_warnings.setFormatter(formatter)

sh_api_info = logging.StreamHandler(sys.stdout)
sh_api_info.setLevel(logging.INFO)
sh_api_info.setFormatter(formatter_out_api)

logger_api = logging.getLogger("{0}.api".format(os.path.basename(_current_dir)))
logger_api.addHandler(fh_api)
logger_api.addHandler(fh_api_warnings)
logger_api.addHandler(sh_api_info)



from phanterpwa.helpers import (
    CONCATENATE
)

text = CONCATENATE(
    "You have created an account on {{app_name}}. Confirm your mobile number, use the activation code:",
    "\n\n",
    "{{code}}",
    "\n\n",
    "Thank you! Bye!"
)



import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers
import plugins.codemirror as codemirror
from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = M = js_undefined = window =\
    __new__ = FormData = console = localStorage = document = this = 0
__pragma__('noskip')

DIV = helpers.XmlConstructor.tagger("div")
LABEL = helpers.XmlConstructor.tagger("label")
I = helpers.XmlConstructor.tagger("i")
H2 = helpers.XmlConstructor.tagger("h2")
FORM = helpers.XmlConstructor.tagger("form")
SPAN = helpers.XmlConstructor.tagger("span")
IMG = helpers.XmlConstructor.tagger("img", True)
HR = helpers.XmlConstructor.tagger("hr", True)
UL = helpers.XmlConstructor.tagger("ul")
STRONG = helpers.XmlConstructor.tagger("strong")
LI = helpers.XmlConstructor.tagger("li")
INPUT = helpers.XmlConstructor.tagger("input", True)
BR = helpers.XmlConstructor.tagger("br", True)
P = helpers.XmlConstructor.tagger("p")
A = helpers.XmlConstructor.tagger("a")
I18N = helpers.I18N
CONCATENATE = helpers.CONCATENATE
XML = helpers.XML
XSECTION = helpers.XSECTION



text1 = """
All objects can be incorporated into Web pages, for this it is only necessary to use the <strong>html_to</strong>, <strong>append_to</strong> or <strong>insert_to</strong> methods, they can also be merged with the html helpers, directly or through variables, below an example of how to use and the necessary imports.
"""
text1_pt_br = """
Todos os objetos podem ser incorporados em páginas da Web, para isto é só usar os métodos <strong>html_to</strong>, <strong>append_to</strong> ou <strong>insert_to</strong>, tambem podem ser mesclados com os helpers html, diretamente ou através de variáveis, abaixo um exemplo de como usar e os imports necessários.
"""
first_example1 = """import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers

DIV = helpers.XmlConstructor.tagger("div")

helper_instance = DIV(
    widgets.Input(
        "android",
        label="Label Widget",
        placeholder="Placeholder Widget",
    ),
    _class="widget_input_example"
)
helper_instance.html_to("#your-container")

new_widget_instance = widgets.Input(
    "other_identifier",
    label="Label Widget"
)
new_widget_instance.append_to("#your-container")

using_jquery = widgets.Input(
    "other_identifier",
    label="Label Widget"
)

element_jquery = using_jquery.jquery()
jQuery("#your-container").append(element_jquery)

"""

input1_source = """import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers

DIV = helpers.XmlConstructor.tagger("div")

helper_instance = DIV(
    widgets.Input(
        "android",
        label="Label Widget",
        placeholder="Placeholder Widget",
    ),
    _class="widget_input_example"
)
helper_instance.html_to("#your-container")
"""

input2_source = """import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers

DIV = helpers.XmlConstructor.tagger("div")

helper_instance = DIV(
    widgets.Input(
        "shadows",
        label="Label Widget",
        placeholder="Placeholder Widget",
        wear="shadows"
    ),
    _class="widget_input_example"
)
helper_instance.html_to("#your-container")
"""

input3_source = """import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers

DIV = helpers.XmlConstructor.tagger("div")

helper_instance = DIV(
    widgets.Input(
        "elegant",
        label="Label Widget",
        placeholder="Placeholder Widget",
        wear="elegant"
    ),
    _class="widget_input_example"
)
helper_instance.html_to("#your-container")
"""

input4_source = """
# your imports
# your code

your_instance = widgets.Input(
    "password",
    label="Label Widget",
    placeholder="Placeholder Widget",
    kind="password"
)

# your code
"""

input5_source = """
# your code

your_instance = widgets.Input(
    "date",
    label="Label Widget",
    placeholder="Placeholder Widget",
    kind="date"
)

# your code
"""

input6_source = """
# your code

your_instance = widgets.Input(
    "datetime",
    label="Label Widget",
    value="2020-02-02 03:12:33",
    placeholder="Placeholder Widget",
    kind="datetime"
)

# your code
"""

input7_source = """
# your code

your_instance = widgets.Input(
    "datetime2",
    label="Label Widget",
    value="02/02/2020 03:12:33",
    placeholder="Placeholder Widget",
    kind="datetime",
    format="dd/MM/yyyy HH:mm:ss"
)

# your code
"""

input8_source = """
# your code

your_instance = widgets.Input(
    "iconify",
    label="Label Widget",
    placeholder="Placeholder Widget",
    icon=I(_class="fab fa-sistrix"),
    icon_on_click=lambda: window.PhanterPWA.flash(**{"html": I18N("Fui Clicado!")})
)

# your code
"""
input_8_source = """
# your code

your_instance = widgets.Input(
    "iconify_password",
    label="Label Widget",
    value="password",
    kind="password",
    placeholder="Placeholder Widget",
    icon=I(_class="fab fa-sistrix")
)

# your code
"""
input9_source = """
# your code

your_instance = widgets.Input(
    "iconify_datetime",
    label="Label Widget",
    placeholder="Placeholder Widget",
    kind="datetime",
    icon=I(_class="far fa-calendar-alt")
)

# your code
"""

input10_source = """
# your code

your_instance = widgets.Input(
    "iconify_date",
    label="Label Widget",
    placeholder="Placeholder Widget",
    kind="date",
    icon=I(_class="far fa-calendar-alt"),
    format="dd/MM/yyyy"
)

# your code
"""


class Inputs():
    def __init__(self, gatehandler):
        self.gatehandler = gatehandler
        gatehandler.Inputs = self
        html = DIV(
            DIV(
                P(I18N(text1, **{"_pt-BR": text1_pt_br, "sanitize": False})),

                H2(I18N("Styles (waer)")),
                HR(),
                XSECTION(
                    LABEL(I18N("Example"), ' 01 (wear="android")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirror1",
                            **{
                                "value": input1_source,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Input(
                                "android",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 02 (wear="shadows")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirror2",
                            **{
                                "value": input2_source,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Input(
                                "shadows",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                wear="shadows"
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 03 (wear="elegant")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirror3",
                            **{
                                "value": input3_source,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Input(
                                "elegant",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                wear="elegant"
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                H2(I18N("Types (kind)")),
                HR(),
                XSECTION(
                    LABEL(I18N("Example"), ' 01 (kind="password")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirror4",
                            **{
                                "value": input4_source,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Input(
                                "password",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                kind="password"
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 02 (kind="date")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirror5",
                            **{
                                "value": input5_source,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Input(
                                "date",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                kind="date"
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 03 (kind="datetime")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirror6",
                            **{
                                "value": input6_source,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Input(
                                "datetime",
                                label="Label Widget",
                                value="2020-02-02 03:12:33",
                                placeholder="Placeholder Widget",
                                kind="datetime"
                            ),
                            _class="widget_input_example"
                        ),
                        BR(),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirror7",
                            **{
                                "value": input7_source,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ' (format="dd/MM/yyyy HH:mm:ss"):'),
                        DIV(
                            widgets.Input(
                                "datetime2",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                kind="datetime",
                                value="02/02/2020 03:12:33",
                                format="dd/MM/yyyy HH:mm:ss"
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                H2(I18N("Iconify (icon)")),
                HR(),
                XSECTION(
                    LABEL(I18N("Example"), ' 01 (icon=I(_class="fab fa-sistrix"))'),

                    DIV(
                        widgets.Input(
                            "iconify",
                            label="Label Widget",
                            placeholder="Placeholder Widget",
                            icon=I(_class="fab fa-sistrix"),
                            icon_on_click=lambda: window.PhanterPWA.flash(**{"html": I18N("Fui Clicado!")})
                        ),
                        _class="widget_input_example"
                    ),
                    codemirror.CodeMirrorHelper(
                        "codemirror8",
                        **{
                            "value": input8_source,
                            "mode": "python",
                            "lineNumbers": True
                        }
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 02 (kind="password", icon=I(_class="fas fa-eye"))'),

                    DIV(
                        widgets.Input(
                            "iconify_password",
                            label="Label Widget",
                            value="password",
                            placeholder="Placeholder Widget",
                            kind="password",
                            icon=I(_class="fas fa-eye")
                        ),
                        _class="widget_input_example"
                    ),
                    codemirror.CodeMirrorHelper(
                        "codemirror_8",
                        **{
                            "value": input_8_source,
                            "mode": "python",
                            "lineNumbers": True
                        }
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 03 (I(_class="far fa-calendar-alt") e kind="datetime"))'),
                    P(
                        I18N(
                            "Neste exemplo adicionamos um ícone de calendário, observe que o parâmatro kind é datetime e não iremos adicionar uma ação ao botão, clique e veja o que acontece."
                        )
                    ),
                    DIV(
                        widgets.Input(
                            "iconify_datetime",
                            label="Label Widget",
                            placeholder="Placeholder Widget",
                            kind="datetime",
                            icon=I(_class="far fa-calendar-alt")
                        ),
                        _class="widget_input_example"
                    ),
                    codemirror.CodeMirrorHelper(
                        "codemirror9",
                        **{
                            "value": input9_source,
                            "mode": "python",
                            "lineNumbers": True
                        }
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 04 (I(_class="far fa-calendar-alt"), kind="date") e format="dd/MM/yyyy")'),
                    P(
                        I18N(
                            "Como você pode observar, com o kind='date', 'icon' definido e sem uma ação para o icon automaticamente foi utilizado o datetimepicker, agora vamos tentar com o date colocando format='dd/MM/yyyy'."
                        )
                    ),
                    DIV(
                        widgets.Input(
                            "iconify_date",
                            label="Label Widget",
                            placeholder="Placeholder Widget",
                            kind="date",
                            icon=I(_class="far fa-calendar-alt"),
                            format="dd/MM/yyyy",
                            value="01/03/1980"
                        ),
                        _class="widget_input_example"
                    ),
                    codemirror.CodeMirrorHelper(
                        "codemirror10",
                        **{
                            "value": input10_source,
                            "mode": "python",
                            "lineNumbers": True
                        }
                    ),
                ),
                _class="card e-padding_20"
            ),
            _class="new-container"
        )
        html.html_to("#input-container-example")



# -*- coding: utf-8 -*-
import json
import os
import glob


class Translator(object):
    """docstring for Translator"""
    instances = dict()

    def __init__(self, path, identifier, debug=False, direct_translation=None):
        super(Translator, self).__init__()
        if not os.path.exists(path):
            os.makedirs(path, exist_ok=True)
        if not os.path.exists(os.path.join(path, "entries.json")):
            with open(os.path.join(path, "entries.json"), 'w', encoding='utf-8') as f:
                json.dump([], f)
        self.keys = set()
        self._languages = {}
        self.debug = debug
        self.path = path
        self.direct_translation = direct_translation
        self.load()
        self.add_instance(identifier, self)

    @classmethod
    def get_instance(cls, identifier):
        return cls.instances.get(identifier, None)

    @classmethod
    def add_instance(cls, identifier, obj):
        cls.instances[identifier] = obj

    def add_language(self, lang):
        """Adds new dictionarie"""
        if not os.path.exists(os.path.join(self.path, "{0}.json".format(lang))):
            with open(os.path.join(self.path, "{0}.json".format(lang)), 'w', encoding='utf-8') as f:
                json.dump({}, f, ensure_ascii=False, indent=2)
                self._languages[lang] = {}

    def load(self):
        """Read dictionaries"""
        if os.path.isdir(self.path):
            for g in glob.glob(os.path.join(self.path, "*.json")):
                if os.path.basename(g) == "entries.json":
                    with open(g, 'r', encoding='utf-8') as f:
                        self.keys = set(json.load(f))
                else:
                    basename = os.path.basename(g)[0:-5]
                    try:
                        with open(g, 'r', encoding='utf-8') as f:
                            self._languages[basename] = json.load(f)
                    except json.decoder.JSONDecodeError as e:
                        raise Exception(e, "file: {g}")
            for l in self._languages:
                for w in self._languages[l]:
                    if w not in self.keys:
                        self.keys.add(w)
            for t in self.keys:
                for l in self._languages:
                    if t not in self._languages[l]:
                        self._languages[l][t] = t
        else:
            raise IOError("path must be a folder. give: {0}".format(self.path))

    @property
    def direct_translation(self):
        return self._direct_translation

    @direct_translation.setter
    def direct_translation(self, dictionary):
        if isinstance(dictionary, str):
            self._direct_translation = str(dictionary)
        elif dictionary is None:
            self._direct_translation = None
        else:
            raise TypeError("The dictionary of T must be a string or None. Given: {0}".format(type(dictionary)))

    def T(self, entry):
        if self.direct_translation:
            return self.translator(entry, self.direct_translation)
        else:
            return self._T(entry)

    def _T(self, entry):
        """Adds new word in all dictionaries after save if it does not exist"""
        if self.debug:
            self.load()
        if entry not in self.keys:
            self.keys.add(entry)
        for t in self.keys:
            for l in self._languages:
                if t not in self._languages[l]:
                    self._languages[l][t] = t
        if self.debug:
            self.save()
        return entry

    def save(self):
        """Save the translations in all dictionaries"""
        sorted_keys = [x for x in self.keys if x]
        sorted_keys.sort()
        t_lang = {}
        for l in self._languages:
            t_lang[l] = {}
            for t in sorted_keys:
                if t not in self._languages[l]:
                    t_lang[l][t] = t
                else:
                    t_lang[l][t] = self._languages[l][t]

        with open(os.path.join(self.path, "entries.json"), 'w', encoding='utf-8') as f:
            json.dump(sorted_keys, f, ensure_ascii=False, indent=2)
        for x in self._languages:
            with open(os.path.join(self.path, "%s.json" % x), 'w', encoding='utf-8') as f:
                json.dump(t_lang[x], f, ensure_ascii=False, indent=2)

    def translate(self, dictionary, entry, translation):
        """creates a translation of a word for a specific dictionary,
        while for others hold the original word.
        If the dictionary or entry does not exist it will be created.

        @dictionary: Name of the dictionary.
        @entry: Word to be translated.
        @translation: translation of the word.

        eg.
        >>> TranlatorInstance.translate("pt-BR", "Orange", "Laranja")

        returns: {"pt-BR", {"Orange": "Laranja"}}
        This translation will be added to this specific dictionary,
        while in the other dictionary will not be changed

        """
        self._T(entry)
        if dictionary not in self._languages:
            self.add_language(dictionary)
            self._languages[dictionary] = {entry: translation}
        else:
            self._languages[dictionary][entry] = translation
        if self.debug:
            self.save()
        return {dictionary: {entry: translation}}

    def translator(self, entry, dictionary=None):
        self._T(entry)
        if dictionary and dictionary in self._languages:
            if entry in self._languages[dictionary]:
                return self._languages[dictionary][entry]
        if self.debug:
            self.save()
        return entry

    def dictionaries(self, entries=None):
        if self.debug:
            self.load()
        if entries:
            if isinstance(entries, (list, tuple)):
                langs = {}
                for l in self.languages:
                    for v in entries:
                        self._T(v)
                        if v in self.languages[l]:
                            langs[l] = {v: self.languages[l][v]}
                return langs
            elif isinstance(entries, dict):
                langs = {}
                for l in self.languages:
                    for v in entries:
                        self._T(entries[v])
                        if entries[v] in self.languages[l]:
                            if l in langs:
                                langs[l][v] = self.languages[l][entries[v]]
                            else:
                                langs[l] = {v: self.languages[l][entries[v]]}
                        else:
                            langs[l] = {v: entries[v]}
                return langs
            elif isinstance(entries, str):
                langs = {}
                for l in self.languages:
                    self._T(entries)
                    if entries in self.languages[l]:
                        langs[l] = {entries: self.languages[l][entries]}
                return langs
            else:
                return {}
        else:
            return self.languages

    def phanterpwa_i18n(self, entrie):
        if isinstance(entrie, str):
            self._T(entrie)
            langs = {}
            o_keys = list(self.languages.keys())
            o_keys.sort()
            for l in o_keys:
                if entrie in self.languages[l]:
                    if entrie != self.languages[l][entrie]:
                        langs[l] = {entrie: self.languages[l][entrie]}
            if langs:
                return langs
            return None
        else:
            raise ('The arg entrie in method phanterpwa_i18n must be String. Given: {0}'.format(type(entrie)))

    @property
    def languages(self):
        return self._languages

    def __bool__(self):
        return os.path.exists(os.path.join(self.path, "entries.json"))


def browser_language(header_language, default="en-US"):
    if header_language:
        languages = header_language.split(",")
        locales = []
        for language in languages:
            parts = language.strip().split(";")
            if len(parts) > 1 and parts[1].startswith("q="):
                try:
                    score = float(parts[1][2:])
                except (ValueError, TypeError):
                    score = 0.0
            else:
                score = 1.0
            locales.append((parts[0], score))
        if locales:
            locales.sort(key=lambda pair: pair[1], reverse=True)
            codes = [l[0] for l in locales]
            if len(codes) > 0:
                return codes[0]
    return default



from datetime import (
    datetime,
    timedelta
)
from passlib.hash import pbkdf2_sha512
from phanterpwa.tools import check_activation_code


class IS_ACTIVATION_CODE:
    def __init__(self, size=6, error_message='error'):
        self.e = error_message
        self.s = size

    def __call__(self, value, record_id=None):
        if check_activation_code(value, self.s):
            return (value, None)
        return (value, self.e)

    def formatter(self, value):
        return value


class HASH_MATCH_WITH_PASSWORD:
    def __init__(self, password, error_message='password does not mach with hash'):
        self.e = error_message
        self.password = password

    def __call__(self, value, record_id=None):
        if pbkdf2_sha512.verify(value, self.password):
            return (value, None)
        return (value, self.e)

    def formatter(self, value):
        return value


class PASSWORD_MATCH_WITH_HASH:
    def __init__(self, hash_password, error_message='password does not mach with hash'):
        self.e = error_message
        self.hash_password = hash_password

    def __call__(self, value, record_id=None):
        if pbkdf2_sha512.verify(value, self.hash_password):
            return (value, None)
        return (value, self.e)

    def formatter(self, value):
        return value

class VALID_PASSWORD:
    def __init__(self, user_record, error_message='password does not mach with hash'):
        self.e = error_message
        self.user_record = user_record

    def __call__(self, value, record_id=None):
        user = self.user_record
        result = pbkdf2_sha512.verify(value, user.password_hash)
        if not result and user.temporary_password_hash:
            result = pbkdf2_sha512.verify(value, user.temporary_password_hash)
        if result:
            return (value, None)    
        return (value, value)

    def formatter(self, value):
        return value


import os
from ...helpers import DIV

PRELOADER = DIV(
    DIV(
        DIV(
            DIV(
                DIV(
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_circle_clipper left'
                    ),
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_gap-patch'
                    ),
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_circle_clipper right'
                    ),
                    _class='spinner-layer spinner-one'
                ),
                DIV(
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_circle_clipper left'
                    ),
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_gap-patch'
                    ),
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_circle_clipper right'
                    ),
                    _class='spinner-layer spinner-two'
                ),
                DIV(
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_circle_clipper left'
                    ),
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_gap-patch'
                    ),
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_circle_clipper right'
                    ),
                    _class='spinner-layer spinner-three'
                ),
                DIV(
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_circle_clipper left'
                    ),
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_gap-patch'
                    ),
                    DIV(
                        DIV(
                            _class='phanterpwa_circle'
                        ),
                        _class='phanterpwa_circle_clipper right'
                    ),
                    _class='spinner-layer spinner-four'
                ),
                _class='phanterpwa_android'
            ),
            _class='preloader-wrapper enabled'
        ),
        _class="preload-wrapper"),
    _class="phanterpwa-components-preloaders-android"
)

PRELOADER.sass_file(
    os.path.join(os.path.dirname(__file__), "android.sass")
)
PRELOADER.sass_vars = {
    'STROKEWIDTH': '10px',
    'CONTAINERWIDTH': '200px',
    'COLOR1': 'blue',
    'COLOR2': 'red',
    'COLOR3': '#f4b400',
    'COLOR4': 'green',
}



import phanterpwa.frontend.gatehandler as gatehandler
import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.components.left_bar as left_bar
import phanterpwa.frontend.components.snippets as snippets
from org.transcrypt.stubs.browser import __pragma__
__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = window = 0
__pragma__('noskip')

DIV = helpers.XmlConstructor.tagger("div")
I = helpers.XmlConstructor.tagger("i")
H1 = helpers.XmlConstructor.tagger("h1")
H2 = helpers.XmlConstructor.tagger("h2")
H3 = helpers.XmlConstructor.tagger("h3")
A = helpers.XmlConstructor.tagger("a")
I18N = helpers.I18N
__pragma__('kwargs')


class Index(gatehandler.Handler):
    def initialize(self):
        html = DIV(
            snippets.Centralizer(
                "phanterpwa-logo-wrapper",
                DIV(_class="phanterpwa-background-logo"),
                H1("PhanterPWA", _class="phanterpwa-the_title"),
                H3(
                    I18N(
                        "Full-Stack Progressive Web Applications framework written and programmable with Python.",

                    ),
                    _class="phanterpwa-the_subtitle"
                )
            )
        )
        html.html_to("#main-container")
        window.PhanterPWA.LOAD(**{
            "args": ["loads", "phanterpwa_logo.html"],
            "onComplete": self._after_load
        })
        AdminButton = left_bar.LeftBarButton(
            "phanterpwa-developer-button",
            "Development",
            I(_class="fas fa-users-cog"),
            **{
                "_phanterpwa-way": "developer",
                "position": "top",
                "ways": ["home"]
            }
        )
        teste = left_bar.LeftBarButton(
            "phanterpwa-components-button",
            "Components",
            I(_class="fas fa-code"),
            **{
                "_phanterpwa-way": "examples",
                "position": "top",
                "ways": ["home"]
            }
        )

        window.PhanterPWA.Components['left_bar'].add_button(AdminButton)
        window.PhanterPWA.Components['left_bar'].add_button(teste)

    def _after_load(self, data):
        xml = jQuery(
                "#phanterpwa-snippet-phanterpwa-logo-wrapper")
        xml.find(".phanterpwa-background-logo").html(data)
        xml.height(jQuery(window).height() - 60).css("width", "100%")
        jQuery(window).resize(lambda: xml.height(jQuery(window).height() - 60).css("width", "100%"))

__pragma__('nokwargs')
